%Estimation of State Capacity Spillovers: Conditional MLE with
%predetermined national choices
%Acemoglu, Garcia-Jimeno, Robinson

clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Import data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Open data file State_Capacity_Network_Work.mat
load State_Capacity_Network_Work.mat

%%
%OPTIONS
                                      
%1: Choose if control for rivers density and land shares
 river_land=0;      %0=no
                    %1=yes

%2: Choose if control for distance to current road
current_road=1;     %0=no
                    %1=yes
                    
%3: Choose assumption on Network Links
 ass_network_structure=0;       %0 = Network link is neighbors           
                                %1 = Network link is neighbors and neighbors of neighbors 

%4: Choose order of instruments: 
 order=2;           %2 = Instruments are Nc and N2c
                    %3 = Instruments are N2c and N3c 

%5: Choose the subset of Instruments to use:
 instruments=0;     %0 = Royal roads and Colonial Agencies
                    %1 = Royal roads only
                    %2 = Colonial Agencies only

%6: Choose if exclude violent municipalities
excl_viol=0;        %0 = Do not exclude
                    %1 = Exclude from sample
                    %2 = Exclude from network
                    
%%

 %Choose the endogenous and exogenous variables for the model
 Yg_input = [Yg(:,2) ((Yg(:,4)+Yg(:,5)+Yg(:,6))/3) Yg(:,9) Yg(:,10)];                   %Prosperity Outcomes: Secondary, Public Utilities, Not in Poverty, LQI
         
     
 S_input = [S(:,6)+S(:,7)+S(:,8)+S(:,11)+S(:,12)+S(:,13)+S(:,14)+S(:,15)+S(:,16)+S(:,18)+S(:,19) S(:,2)];                 %S = Contemporary State
           
     C_input = [C(:,1) C(:,3)];                                               %C = Colonial State
     R_input = R;                                                             %R = Distance to royal roads
     
     %Choose controls
     if river_land==0 && current_road==0                        %X = Controls
         X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5)];
        else if river_land==0 && current_road==1
                X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1)];               
            else if river_land==1 && current_road==0
                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) G(:,6:8) L(:,1:7) L(:,12:15)];
                else if river_land==1 && current_road==1
                        X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) G(:,6:8) L(:,1:7) L(:,12:15)];
                    end
                end
            end
     end
                   
     
     %Ev_input = Ev;                                                          %Historical Electoral variability
     A_input = Adj;                                                           %A = Adjacency matrix
     Dd_input = Ddist;                                                        %Original Distances matrix
     Drr_input = Drankrange;                                                  %Original Range of elevation ranks along path matrix
     Drv_input = Drankvar;                                                    %Original Variance of elevation ranks along path matrix
     Yv_input = Yv(:,16);                                                     %Original Per capita attacks     
     
     
     %Clean data from missing values
     [Yg_clean S_clean C_clean X_clean R_clean Yv_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean] = clean_missings(Yg_input, S_input, C_input, X_input, R_input, Yv_input, A_input, Dd_input, Drr_input, Drv_input, DaneCodes);

 clear Adj C DD Dc Ddist Drankrange Drankvar G H L R S Xs Yg Yv Ygini Ev DaneCodes
 clear Yg_input S_input C_input X_input R_input Yv_input Ev_input A_input Dd_input Drr_input Drv_input


 global p l c r b
 p = Yg_clean;                          %p=vectors of prosperity
 
 l= log(1+S_clean(:,1));                %l=vector of LOCAL state capacity  
 b= log(1+S_clean(:,2));                %b=vector of NATIONAL-LEVEL state capacity  
 c = (C_clean);                         %c=vector of colonial state presence
 r = zscore(R_clean);                   %r=vector of distance to royal roads
 
 global D E
 D= Dd_clean;                           %D=distance matrix
 E= Drv_clean;                          %E=variance elevation matrix

 global X X_d pop

 %Choose set of controls
   
    X = X_clean(:,28:end);                  %matrix of controls
    X_d = X_clean(:,2:27);                  %matrix of department dummies and department capital dummy
    pop = X_clean(:,1)./1000;               %vector of population
 
 global A
 A=A_clean;                             %A=adjacency matrix (nxn) with 0s on the diagonal

 global n k ndep
 n=size(p,1);                           %number of observations
 k=size(X,2);                           %number of covariates
 ndep=size(X_d,2);                      %number of department dummies
 
 DaneCodes=DaneCodes_clean;             %Municipality codes to keep track
 
 %Create Indicator of violent municipalities (above 90th percentile of either paramilitary of guerrilla attacks)
 v=zeros(n,1);
 pc90 = prctile(Yv_clean,90);
 for j=1:n
     if Yv_clean(j,1)>=pc90
         v(j)=1;
     end
 end
 
 clear Yg_clean S_clean C_clean X_clean R_clean Yv_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean
 
 
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Assumptions on the Network Structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if excl_viol==2
    p = p(~v,:);
    l = l(~v,:);
    b = b(~v,:);
    c = c(~v,:);
    r = r(~v,:);
    X = X(~v,:);
    X_d =X_d(~v,:);
    pop = pop(~v,:);
    
    D0 = D(~v,:)';
    D = (D0(~v,:))';
    
    E0 = E(~v,:)';
    E = (E0(~v,:))';
    
    A0 = A(~v,:)';
    A = (A0(~v,:))';

    clear D0 E0 A0

    n=size(p,1);
end

global NE N N_2 A_2 A_3
F=1./(1+(D.*(1+E)));

%Re-sets the diagonal to zeroes
for i=1:size(F,1)
    F(i,i)=0;
end

Q=(ones(n,n)-A).*(A*A-diag(diag(A*A)));                 %Matrix of Number of Paths to Neighbors of Neighbors who are not own Neighbors
A_2=ismember(Q,1:1:max(max(Q)));                        %Adjacency Matrix of neighbors of neighbors
A_2=+A_2;                                               % from logical to numeric

Q2=(ones(n,n)-A-A_2).*((A*A*A)-diag(diag(A*A*A)));      %Matrix of Number of Paths to Neighbors of Neighbors of Neighbors who are not own Neighbors or own Neighbors of Neighbors
A_3 = ismember(Q2,1:1:max(max(Q2)));                    %Adjacency Matrix of neighbors of neighbors of neighbors
A_3=+A_3;                                               % from logical to numeric
clear Q Q2


if ass_network_structure==1 
    A=(A+A_2);                                          %Adjacency matrix when Network link is defined by assumption b
    A_2=A_3;                                            %Adjacency matrix of neighbors of neighbors when Network link is defined by assumption b
end

%Network matrix of adjacent neighbors
NE = F.*A; 

if order==2
    N=F.*A;                                                 %Network matrix of neighbors
    N_2=F.*A_2;                                             %Network matrix of neighbors of neighbors
else if order==3
        N=F.*A_2;                                           %Network matrix of neighbors of neighbors
        N_2=F.*A_3;                                         %Network matrix of neighbors of neighbors of neighbors
    end
end

%% Conditional Maximum Likelihood

assum_f_delta=0;
linear_case=0;

% Four prosperity outcomes
global num_out
num_out=size(p,2);


likelihood=zeros(5,5,5);
params_mle=zeros(5,5,5,184);
itera=0;
for ii=1:5
    for jj=1:5
        for kk=1:5
      
      theta = 0.2*ii;
      psi_1 = 0.5;
      psi_2 = ones(1,num_out);
      beta_out = zeros(1,num_out*(k+1));
      zet_out = zeros(1,num_out*ndep);
      phi = 0.1*ones(1,3);
      beta_br = zeros(1,(k+1));
      zet_br = zeros(1,ndep);
      sigma = 0.5 + jj*0.2;
      alpha = 0;          
      stds = 10*ones(1,num_out+1);
      tau = 0.3*kk;
      delta = 1*ones(1,2);
      

% Initial guess for the parameters
guess=[theta psi_1 psi_2 beta_out zet_out phi beta_br zet_br sigma alpha stds tau delta];

num_param = length(guess);

lb=[0   -Inf    0*ones(1,num_out) -Inf*ones(1,num_out*(k+1)) -Inf*ones(1,num_out*ndep) -Inf*ones(1,3) -Inf*ones(1,k+1) -Inf*ones(1,ndep)   0  -Inf   0*ones(1,num_out+1)   0   0   0 ];
ub=[Inf  Inf  Inf*ones(1,num_out)  Inf*ones(1,num_out*(k+1))  Inf*ones(1,num_out*ndep)  Inf*ones(1,3)  Inf*ones(1,k+1)  Inf*ones(1,ndep) Inf   Inf Inf*ones(1,num_out+1) Inf Inf Inf ];

% Optimization
options=optimset('Display','iter','MaxIter',70000,'MaxFunEvals',70000,'Algorithm','interior-point');%,'TolX',1e-10);
tic
[params_mle(ii,jj,kk,:), likelihood(ii,jj,kk)] = fmincon(@(param) cond_mle_objective_function(param, assum_f_delta, order, linear_case), guess, [], [], [], [], lb, ub, [], options);
itera=itera+1;
disp(itera)
toc

        end
    end
end

%Choose the initial values that converge to the smallest likelihood
[min_logL kk] = min(min(min(likelihood)));
zz = reshape(likelihood(:,:,kk),5,5);
[ii jj] = find(zz == min_logL);
lkh=likelihood(ii,jj,kk);

param_mle=reshape(params_mle(ii,jj,kk,:),1,num_param);
num_param=length(param_mle);

if assum_f_delta==0
    param_mle(end-1:end)=ones(1,2);
end

% Display results
theta = param_mle(1);
disp('theta');
disp(theta)

psi_1 = param_mle(2);
disp('psi_1');
disp(psi_1);

psi_2 = param_mle(3:3+num_out-1);
disp('psi_2')
disp(psi_2)

phi = param_mle(3+num_out+(num_out*(k+1))+(num_out*ndep):3+num_out+(num_out*(k+1))+(num_out*ndep)+3-1);
disp('phi')
disp(phi)

if linear_case==1
    sigma=1;
    
    alpha=1;
else
    sigma = param_mle(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep);
    
    index = param_mle(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1)';
    
    alpha = exp(index)./( 1 + exp(index) );    
end

disp('sigma')
disp(sigma)

disp('alpha')
disp(alpha)

stds = param_mle(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1+1:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1+num_out+1);
disp('Variances')
disp((stds.^2))

tau = param_mle(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1+num_out+1+1);
disp('tau')
disp(tau)

delta = param_mle(end-1:end);
disp('delta')
disp(delta)

beta_br = param_mle(3+num_out+(num_out*(k+1))+(num_out*ndep)+3:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1-1)';
zet_br = param_mle(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1)';
beta_out = param_mle(3+num_out:3+num_out+(num_out*(k+1))-1)';
zet_out = param_mle(3+num_out+(num_out*(k+1)):3+num_out+(num_out*(k+1))+(num_out*ndep)-1)';


% g()
if sigma==1
    s = (l.^alpha).*((tau*b).^(1-alpha));
else
    s = ( (alpha.*(l.^((sigma-1)/sigma))) + ((1-alpha).*((tau*b).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end

%Estimated Network matrix
NN = neighbors_matrix(delta, order);

    Avg_nij = NN(:);
    Avg_nij =  mean(Avg_nij(Avg_nij~=0))

Avg_nij*mean(s)*psi_1 + Avg_nij*psi_2 
   

biweight_kernel=0;
truncation_normal=10;
nobin=0;
yy = (theta./alpha).*l.*((l./s).^(1/sigma)) - psi_1*(NN*s) - X_d*zet_br;
xx = [log(1+c) r zscore(X) log(pop)];
g_hat = nonparametric_conditional( yy, xx, biweight_kernel, truncation_normal, [phi'; beta_br], nobin );

kappa = theta.*g_hat;
E_kappa = theta*mean(g_hat);
disp('E[kappa]')
disp(E_kappa)

stdev_kappa = theta*std(g_hat);
disp('Stdev Kappa')
disp(stdev_kappa)

g_func = [xx*[phi'; beta_br] g_hat];


%%
%Condition for Equilibrium Uniqueness
lambda_min = min(eig(NN));
disp('Lowest Eigenvalue of the Network Matrix')
disp(lambda_min)

TT = ((sigma+1)*((l./s).^(1/sigma))) - (alpha*(l./s)) ;

dl_dNs =(psi_1*alpha*sigma./theta).*(1./TT);

dl_dNs_inv=(dl_dNs).^(-1);

disp('Min (dl_dN(d)s)^(-1)')
disp(min(dl_dNs_inv))

disp('Max (dl_dN(d)s)^(-1)')
disp(max(dl_dNs_inv))

if ( min(dl_dNs_inv)>(1+(1/lambda_min)) ) && ( max(dl_dNs_inv)<1 )
     disp('Unique Equilibrium Condition Satisfied')
 else
     disp('Unique Equilibrium Condition Not Satisfied')
 end

%%
%%%%%%%%%%%%%%%%%%%
%Standard Errors
%%%%%%%%%%%%%%%%%%%


% variables for the likelihood
h_xi = (theta./alpha).*l.*((l./s).^(1/sigma)) - psi_1*(NN*s) - g_hat - X_d*zet_br;

h_eps = zeros(n,num_out);
for j=1:num_out
    h_eps(:,j) = zscore(p(:,j)) - ((theta./alpha).*l.*s.*((l./s).^(1/sigma))) - (psi_2(j).*(NN*s)) - ([zscore(X) pop]*beta_out(((j-1)*k)+j:j*(k+1))) - (X_d*zet_out((1+((j*ndep)-ndep):j*ndep)));
end

U=[h_xi h_eps];

% derivatives of the neighborhood matrix on delta  
T(:,:,1)=-D.*(1+delta(2)*E)./((1+delta(1)*D.*(1+delta(2)*E))).^2;
T(:,:,2)=-D.*delta(1).*E./((1+delta(1)*D.*(1+delta(2)*E))).^2;

% derivatives of the index function g
xxx = [ones(n,1) g_func(:,1)];
bb=(xxx'*xxx)\(xxx'*g_func(:,2));

dgdcoeff=xx.*bb(2);
dgdphi=dgdcoeff(:,1:3);
dgdbeta=dgdcoeff(:,4:end);

% gradient of the matrix U
dUdpsi=zeros(n,num_out+1, num_out+1);
dUdpsi(:,1,1)=-NN*s;
for i=2:num_out+1
    dUdpsi(:,i,i)=-NN*s;
end

dUdbeta=zeros(n,k+1,num_out+1);
dUdbeta(:,:,1)=-dgdbeta; 

dUdphi=zeros(n,3,num_out+1);
dUdphi(:,:,1)=-dgdphi; 

dUddelta=zeros(n,2,num_out+1);

if assum_f_delta==1
    dUddelta(:,1,:)=[-psi_1*T(:,:,1)*s -T(:,:,1)*s*psi_2];
    dUddelta(:,2,:)=[-psi_1*T(:,:,2)*s -T(:,:,2)*s*psi_2];
end

dUdzeta=zeros(n,ndep,num_out+1);
dUdzeta(:,:,1)=-X_d;

dUdtheta=zeros(n,1,num_out+1);
dUdtheta(:,:,1)=(((1./alpha).*l.*(l.*s).^(1/sigma))./((l./s).^(2*(1+sigma))));
dUdtheta(:,:,2:num_out+1)=repmat(-(((1./alpha).*l.*s.*((l.*s).^(1/sigma)))./((l./s).^(2*(1+sigma)))),[1 1 num_out]);

dUdalpha=zeros(n,1,num_out+1);
dUdalpha(:,:,1)=-((theta./alpha).*l.*( ((1/alpha).*((l./s).^(1/sigma))) + ((l - ((tau*b).*((l./(tau*b)).^(1/sigma))))./((sigma-1).*s)))./((l./s).^(2*(1+sigma))));
dUdalpha(:,:,2:num_out+1)=repmat((theta.*(l.^((1+sigma)/sigma)).*( (2.*alpha.*(l.^((sigma-1)/sigma))) + (((tau*b).^((sigma-1)/sigma)).*(1-(2.*alpha))) )./(( ((alpha^2).*(l.^((sigma-1)/sigma))) + (alpha.*(1-alpha).*((tau*b).^((sigma-1)/sigma))) ).^2)./((l./s).^(2*(1+sigma)))), [1 1 num_out]);

deriv_s_over_alpha=(sigma/(sigma-1)).*(s.^(1/sigma)).*( (l.^((sigma-1)/sigma)) - ((tau*b).^((sigma-1)/sigma)) );
deriv_s_over_tau=((1-alpha).*s.^(1/sigma).*b.^((sigma-1)/sigma).*tau.^(-1/sigma))./((l./s).^(2*(1+sigma)));
deriv_s_over_sigma=((alpha.*l.^((sigma-1)/sigma)-(alpha-1).*(b*tau).^((sigma-1)/sigma)).^(sigma/(sigma-1)).*(-sigma.*(-alpha.*l.*(b*tau).^(1/sigma)...
    +alpha.*(b*tau).*l.^(1/sigma)-(b*tau).*l.^(1/sigma)).*log(alpha.*l.^((sigma-1)/sigma)-(alpha-1).*(b*tau).^((sigma-1)/sigma))-alpha.*l.*(sigma-1).*(b*tau).^(1/sigma).*log(l)...
    +(alpha-1).*(b*tau).*(sigma-1).*log((b*tau)).*l.^(1/sigma)))./((sigma-1)^2 .*sigma.*(-alpha.*l.*(b*tau).^(1/sigma)+alpha.*(b*tau).*l.^(1/sigma)-(b*tau).*l.^(1/sigma)));

dUdsigma=zeros(n,1,num_out+1);
dUdsigma(:,:,1)=((theta./alpha).*(l.^((sigma+1)/sigma)).*(-log(sigma).*s.^(-(2+sigma)/sigma)/sigma^3).*deriv_s_over_sigma);
dUdsigma(:,:,2:num_out+1)=repmat(-((theta./alpha).*l.^((1+sigma)/sigma).*(log(sigma).*s.^((sigma-2)/sigma).*(sigma-1)/sigma^3).*deriv_s_over_sigma),[1 1 num_out]);

dUdtau=zeros(n,1,num_out+1);
dUdtau(:,:,1)=((theta./alpha).*(l.^((sigma+1)/sigma)).*(-(1/sigma)*s.^(-(1+sigma)/sigma)).*deriv_s_over_tau);
dUdtau(:,:,2:num_out+1)=repmat(-((theta./alpha).*l.^((1+sigma)/sigma).*(-(1/sigma)*s.^(-(1+sigma)/sigma)).*deriv_s_over_tau),[1 1 num_out]);

dUdbeta_out=zeros(n,(k+1)*num_out,num_out+1);
dUdzet_out=zeros(n,ndep*num_out,num_out+1);
for j=1:num_out
dUdbeta_out(:,((j-1)*k)+j:j*(k+1),2:end)=repmat(-[zscore(X) pop],[1 1 num_out]);
dUdzet_out(:,(1+((j*ndep)-ndep):j*ndep),2:end)=repmat(-X_d,[1 1 num_out]);
end

Gradient_U=[dUdtheta dUdpsi dUdbeta_out dUdzet_out dUdphi dUdbeta dUdzeta dUdsigma dUdalpha dUdtau dUddelta];
mean_Grad_U=mean(Gradient_U);

% Gradient of the inverse of the Variance Covariance matrix 
VarCov=[stds(1)^2 0 0 0 0; 0 stds(2)^2 0 0 0; 0 0 stds(3)^2 0 0; 0 0 0 stds(4)^2 0; 0 0 0 0 stds(5)^2];
VarCov_gradient=-(2*VarCov.^(-3/2));

% Gradient of the log determinant of J
det_J = ((theta./alpha).*(1/sigma).*( ( (1+sigma).*( (l./s).^(1/sigma) ) ) - ( alpha.*(l./s) ) )); 

GradientJ=zeros(n,num_param);
dJdtheta=(((1./alpha).*(1/sigma).*( ( (1+sigma).*( (l./s).^(1/sigma) ) ) - ( alpha.*(l./s) ) )))./(((l./s).^(2*(1+sigma))).*det_J);
dJdalpha=(theta/(alpha*sigma)).*((l./s).^(1/sigma)).*( (((deriv_s_over_alpha)./s).*( (alpha.*((l./s).^((sigma-1)/sigma))) - ((sigma+1)/sigma) )) - ((sigma+1)/alpha) )./((l./s).^(2*(1+sigma)));
dJdsigma=(theta./alpha).*( (2.*(alpha./(sigma^2)).*(l./s)) + ((alpha/sigma).*(l./s).*(deriv_s_over_sigma./s)) + (((l./s).^(1/sigma)).*( (1/sigma) - ((1+sigma)/(sigma^2)) + (((1+sigma)/(sigma^3)).*(log(l./s))) - (((sigma+1)/(sigma^4)).*(deriv_s_over_sigma./s)) ) ) )./(((l./s).^(2*(1+sigma))).*det_J);
dJdtau=(theta./alpha).*(1/sigma).*(deriv_s_over_tau./s).*( (alpha.*(l./s)) - (((1+sigma)/sigma).*((l./s).^(1/sigma))) )./(((l./s).^(0*(1+sigma))).*det_J);

GradientJ(:,1)=(dJdtheta);
GradientJ(:,3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1)=(dJdalpha);
GradientJ(:,3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep)=(dJdsigma);
GradientJ(:,end-2)=(dJdtau);


% Score
dUSUdstds=zeros(n,num_out+1);
dUSUdparam=zeros(n,num_param-num_out-1);
for i=1:n
    for j=1:num_out+1
        dUSUdstds(i,j)= (-2/stds(j)) + VarCov_gradient(j,j)*U(i,j)^2;
    end
    
    for jj=1:num_param-num_out-1
        auxil=0;
        for j=1:num_out+1
            %auxil=2*VarCov_gradient(j,j)*Gradient_U(i,jj,j)+auxil;
             auxil=(2*(U(i,j).*( (Gradient_U(i,jj,j) - mean_Grad_U(1,jj,j)) ) )./VarCov(j,j)) + auxil;
        end
        dUSUdparam(i,jj)=auxil;
    end
end

GradientUSU=[dUSUdparam(:,1:end-3) dUSUdstds dUSUdparam(:,end-2:end)];

Score=-0.5*GradientUSU + GradientJ;

outer_product=zeros(num_param,num_param,n);

for i=1:n
    outer_product(:,:,i)=Score(i,:)'*Score(i,:);
end

Var_mle=sum(outer_product,3)/n;

%Var_mle=(Score'*Score)/n;

std_error_MLE=sqrt(diag(Var_mle)); 

disp('standard errors of MLE')
disp(std_error_MLE)


% Display results
std_theta = std_error_MLE(1);
disp('std error theta');
disp(std_theta)

std_psi_1 = std_error_MLE(2);
disp('std error psi_1');
disp(std_psi_1);

std_psi_2 = std_error_MLE(3:3+num_out-1);
disp('std error psi_2')
disp(std_psi_2)

std_phi = std_error_MLE(3+num_out+(num_out*(k+1))+(num_out*ndep):3+num_out+(num_out*(k+1))+(num_out*ndep)+3-1);
disp('std error phi')
disp(std_phi)

if linear_case==1
    std_sigma=0;
    
    std_alpha=0;
else
    std_sigma = std_error_MLE(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep);
   
    std_alpha = std_error_MLE(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1);    
end

disp('std error sigma')
disp(std_sigma)

disp('std error alpha')
disp(std_alpha)

std_stds = std_error_MLE(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1+1:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1+num_out+1);
disp('std error Variances')
disp((0.5*((1./sqrt(stds)).*std_stds')))

std_tau = std_error_MLE(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1+num_out+1+1);
disp('std error tau')
disp(std_tau)


if assum_f_delta==0
    std_delta =[0 0];
else
    std_delta = std_error_MLE(end-1:end);
end

disp('std error delta')
disp(std_delta)

%%
%Marginal Effects

%dL_i/dL_j in the Best Response
Num = (s./l).^(1/sigma);
Num = repmat(Num,1,n);

Den = ((sigma+1)*((l./s).^(1/sigma))) - (alpha*(l./s));
Den = repmat(Den,1,n);

Quot = Num./Den;

NQ = NN.*Quot;

Avg_NQ = sum(sum(NQ))./sum(sum(A));

d_li_d_lj = (psi_1*alpha*sigma/theta)*Avg_NQ;

disp('Average Non-linear Marginal effect')
disp(d_li_d_lj)

%dp_i/ds_i in the prosperity equation

d_pi_d_si = theta.*((sigma-1)/sigma).*((l./s).^((sigma+1)/sigma));

d_pi_d_si = sum(d_pi_d_si)./n;

disp('Average Non-linear Marginal effect')
disp(d_pi_d_si)


%dp_i/ds_j in the prosperity equation
PR = NN*((s./l).^(1/sigma));
PR = sum(PR)./n;

d_pi_d_sj = alpha*PR*psi_2;

disp('Average Non-linear Marginal effect')
disp(d_pi_d_sj)


%%
%Experiments
%Experiments consist of finding the zero of the system of non-linear best responses after different kinds of shocks. Use Newton-Raphson to find the roots

%Baseline:
l_0 = l.*ones(n,1);

%Shock to national state capacity
b_shock = zeros(n,1);

%Shock to weight of local state
alpha_sh = alpha;

%Shock to elasticity of substitution
sigma_sh = sigma ;

%Shock to marginal cost of local state
theta_sh = theta;

%Shock to interaction effect
psi_1_sh = psi_1;

%Shock to elasticity of national state tau
tau_sh = tau;

%Shock to "infrastructure"
delta_sh = 1*delta;
NN_sh = neighbors_matrix(delta_sh, order);

    Avg_nij_pr = NN_sh(:);
    Avg_nij_pr =  mean(Avg_nij_pr(Avg_nij_pr~=0))

    Avg_nij_pr*mean(l)*psi_1 + Avg_nij*psi_2(1)  



BR(alpha_sh, sigma_sh, theta_sh, psi_1_sh, tau_sh, g_hat, zet_br, NN_sh, b_shock, log(l_0))

f = @(x) BR(alpha_sh, sigma_sh, theta_sh, psi_1_sh, tau_sh, g_hat, zet_br, NN_sh, b_shock, x) ; % function of dummy variable x

options = optimset('Display','iter', 'MaxFunEvals', 1000000, 'MaxIter', 1000);
[root, fval] = fsolve(f, log(l_0), options);

BR(alpha_sh, sigma_sh, theta_sh, psi_1_sh, tau_sh, g_hat, zet_br, NN_sh, b_shock, root)

l_base = exp(root);

%Prosperity Outcomes
%Baseline
if sigma_sh==1
    s_base = (l_base.^alpha_sh).*((tau_sh*(b)).^(1-alpha_sh));
else
    s_base = ( (alpha_sh.*(l_base.^((sigma_sh-1)/sigma_sh))) + ((1-alpha_sh).*((tau_sh*(b)).^((sigma_sh-1)/sigma_sh))) ).^(sigma_sh/(sigma_sh-1));
end

p_base=zeros(n,num_out);
p_base_level=zeros(n,num_out);
for zz=1:num_out
    p_base(:,zz) = ( (theta_sh./alpha_sh).*l_base.*s_base.*((l_base./s_base).^(1/sigma_sh)) ) + (psi_2(zz).*(NN_sh*s_base)) + ([zscore(X) pop]*beta_out(((zz-1)*k)+zz:zz*(k+1))) + (X_d*zet_out((1+((zz*ndep)-ndep):zz*ndep)));
    p_base_level(:,zz) =((std(p(:,zz))*p_base(:,zz)) + mean(p(:,zz)));
end






%Experiment: Equalize National State Capacity
l_0 = l.*ones(n,1);
%l_0 = l_predict;

%Shock to national state capacity
b_shock = max(b, median(b))-b;

%Shock to weight of local state
alpha_sh = alpha;

%Shock to elasticity of substitution
sigma_sh = sigma ;

%Shock to marginal cost of local state
theta_sh = theta;

%Shock to interaction effect
psi_1_sh = psi_1;

%Shock to elasticity of national state tau
tau_sh = tau;

%Shock to "infrastructure"
delta_sh = 1*delta;
NN_sh = neighbors_matrix(delta_sh, order);

    Avg_nij_pr = NN_sh(:);
    Avg_nij_pr =  mean(Avg_nij_pr(Avg_nij_pr~=0))

    Avg_nij_pr*mean(l)*psi_1 + Avg_nij*psi_2(1)  



BR(alpha_sh, sigma_sh, theta_sh, psi_1_sh, tau_sh, g_hat, zet_br, NN_sh, b_shock, log(l_0))

f = @(x) BR(alpha_sh, sigma_sh, theta_sh, psi_1_sh, tau_sh, g_hat, zet_br, NN_sh, b_shock, x) ; % function of dummy variable x

options = optimset('Display','iter', 'MaxFunEvals', 1000000, 'MaxIter', 1000);
[root, fval] = fsolve(f, log(l_0), options);

BR(alpha_sh, sigma_sh, theta_sh, psi_1_sh, tau_sh, g_hat, zet_br, NN_sh, b_shock, root)

l_predict = exp(root);

%Effect of Experiment

[l_base l_predict];

scatter(getcolumn(ans(1:length(l),1:2),1),getcolumn(ans(1:length(l),1:2),2),'DisplayName','ans(1:length(l),1:2)(:,2) vs. ans(1:length(l),1:2)(:,1)','YDataSource','ans(1:length(l),1:2)(:,2)');figure(gcf)

median(exp(l)-1)
median(exp(l_base)-1)
median(exp(l_predict)-1)

Delta_l=median(exp(l)-1)*(1+((median(exp(l_predict)-1) - median(exp(l_base)-1))./median(exp(l_base)-1)))

disp('percent change in l')
disp(100*((median(exp(l_predict)-1) - median(exp(l_base)-1))./median(exp(l_base)-1)))


%Prosperity Outcomes
%Baseline
if sigma_sh==1
    s_base = (l_base.^alpha_sh).*((tau_sh*(b)).^(1-alpha_sh));
else
    s_base = ( (alpha_sh.*(l_base.^((sigma_sh-1)/sigma_sh))) + ((1-alpha_sh).*((tau_sh*(b)).^((sigma_sh-1)/sigma_sh))) ).^(sigma_sh/(sigma_sh-1));
end

p_base=zeros(n,num_out);
p_base_level=zeros(n,num_out);
for zz=1:num_out
    p_base(:,zz) = ( (theta_sh./alpha_sh).*l_base.*s_base.*((l_base./s_base).^(1/sigma_sh)) ) + (psi_2(zz).*(NN_sh*s_base)) + ([zscore(X) pop]*beta_out(((zz-1)*k)+zz:zz*(k+1))) + (X_d*zet_out((1+((zz*ndep)-ndep):zz*ndep)));
    p_base_level(:,zz) =((std(p(:,zz))*p_base(:,zz)) + mean(p(:,zz)));
end


if sigma_sh==1
    s_predict = (l_predict.^alpha_sh).*((tau_sh*(b + b_shock)).^(1-alpha_sh));
else
    s_predict = ( (alpha_sh.*(l_predict.^((sigma_sh-1)/sigma_sh))) + ((1-alpha_sh).*((tau_sh*(b + b_shock)).^((sigma_sh-1)/sigma_sh))) ).^(sigma_sh/(sigma_sh-1));
end

p_predict=zeros(n,num_out);
p_predict_level=zeros(n,num_out);
for zz=1:num_out
    p_predict(:,zz) = ( (theta_sh./alpha_sh).*l_predict.*s_predict.*((l_predict./s_predict).^(1/sigma_sh)) ) + (psi_2(zz).*(NN_sh*s_predict)) + ([zscore(X) pop]*beta_out(((zz-1)*k)+zz:zz*(k+1))) + (X_d*zet_out((1+((zz*ndep)-ndep):zz*ndep)));
    p_predict_level(:,zz) = ((std(p(:,zz))*p_predict(:,zz)) + mean(p(:,zz)));
end


Delta_p=zeros(1,num_out);
for zz=1:num_out
    Delta_p(1,zz) = ((median(p_predict_level(:,zz)) - median(p_base_level(:,zz)))./median(p_base_level(:,zz)));
end

p_experiment=zeros(1,num_out);

for zz=1:num_out
    p_experiment(1,zz) = (1+Delta_p(1,zz)).*median(p(:,zz));
end

disp('Observed median prosperity')
disp(median(p))
disp('Counterfactual median prosperity')
disp(p_experiment)



%Experiment: Take below-median local state capacity to median
l_0 = max(median(l),l);

%Shock to national state capacity
b_shock = b;

%Shock to weight of local state
alpha_sh = alpha;

%Shock to elasticity of substitution
sigma_sh = sigma ;

%Shock to marginal cost of local state
theta_sh = theta;

%Shock to interaction effect
psi_1_sh = psi_1;

%Shock to elasticity of national state tau
tau_sh = tau;

%Shock to "infrastructure"
delta_sh = 1*delta;
NN_sh = neighbors_matrix(delta_sh, order);

    Avg_nij_pr = NN_sh(:);
    Avg_nij_pr =  mean(Avg_nij_pr(Avg_nij_pr~=0))

    Avg_nij_pr*mean(l)*psi_1 + Avg_nij*psi_2(1)  

    
%General Equilibrium Effect
BR(alpha_sh, sigma_sh, theta_sh, psi_1_sh, tau_sh, g_hat, zet_br, NN_sh, b_shock, log(l_0))

f = @(x) BR(alpha_sh, sigma_sh, theta_sh, psi_1_sh, tau_sh, g_hat, zet_br, NN_sh, b_shock, x) ; % function of dummy variable x

options = optimset('Display','iter', 'MaxFunEvals', 1000000, 'MaxIter', 1000);
[root, fval] = fsolve(f, log(l_0), options);

BR(alpha_sh, sigma_sh, theta_sh, psi_1_sh, tau_sh, g_hat, zet_br, NN_sh, b_shock, root)

l_predict = exp(root);

%Effect of Experiment

[l_base l_predict l_predict./l_base];

scatter(getcolumn(ans(1:length(l),1:2),1),getcolumn(ans(1:length(l),1:2),2),'DisplayName','ans(1:length(l),1:2)(:,2) vs. ans(1:length(l),1:2)(:,1)','YDataSource','ans(1:length(l),1:2)(:,2)');figure(gcf)

median(exp(l)-1)
median(exp(l_base)-1)
median(exp(l_predict)-1)

Delta_l=median(exp(l)-1)*(1+((median(exp(l_predict)-1) - median(exp(l_base)-1))./median(exp(l_base)-1)))

disp('percent change in l')
disp(100*((median(exp(l_predict)-1) - median(exp(l_base)-1))./median(exp(l_base)-1)))


%Prosperity Outcomes
%Baseline
if sigma_sh==1
    s_base = (l_base.^alpha_sh).*((tau_sh*(b)).^(1-alpha_sh));
else
    s_base = ( (alpha_sh.*(l_base.^((sigma_sh-1)/sigma_sh))) + ((1-alpha_sh).*((tau_sh*(b)).^((sigma_sh-1)/sigma_sh))) ).^(sigma_sh/(sigma_sh-1));
end

p_base=zeros(n,num_out);
p_base_level=zeros(n,num_out);
for zz=1:num_out
    p_base(:,zz) = ( (theta_sh./alpha_sh).*l_base.*s_base.*((l_base./s_base).^(1/sigma_sh)) ) + (psi_2(zz).*(NN_sh*s_base)) + ([zscore(X) pop]*beta_out(((zz-1)*k)+zz:zz*(k+1))) + (X_d*zet_out((1+((zz*ndep)-ndep):zz*ndep)));
    p_base_level(:,zz) =((std(p(:,zz))*p_base(:,zz)) + mean(p(:,zz)));
end


if sigma_sh==1
    s_predict = (l_predict.^alpha_sh).*((tau_sh*(b + b_shock)).^(1-alpha_sh));
else
    s_predict = ( (alpha_sh.*(l_predict.^((sigma_sh-1)/sigma_sh))) + ((1-alpha_sh).*((tau_sh*(b + b_shock)).^((sigma_sh-1)/sigma_sh))) ).^(sigma_sh/(sigma_sh-1));
end

p_predict=zeros(n,num_out);
p_predict_level=zeros(n,num_out);
for zz=1:num_out
    p_predict(:,zz) = ( (theta_sh./alpha_sh).*l_predict.*s_predict.*((l_predict./s_predict).^(1/sigma_sh)) ) + (psi_2(zz).*(NN_sh*s_predict)) + ([zscore(X) pop]*beta_out(((zz-1)*k)+zz:zz*(k+1))) + (X_d*zet_out((1+((zz*ndep)-ndep):zz*ndep)));
    p_predict_level(:,zz) = ((std(p(:,zz))*p_predict(:,zz)) + mean(p(:,zz)));
end


Delta_p_median=zeros(1,num_out);
for zz=1:num_out
    Delta_p_median(1,zz) = ((median(p_predict_level(:,zz)) - median(p_base_level(:,zz)))./median(p_base_level(:,zz)));
end

p_experiment=zeros(1,num_out);
for zz=1:num_out
    p_experiment(1,zz) = (1+Delta_p(1,zz)).*median(p(:,zz));
end


disp('Observed median prosperity')
disp(median(p))
disp('Counterfactual median prosperity')
disp(p_experiment)



    %Direct Effect    
if sigma_sh==1
    s_direct = (l_0.^alpha_sh).*((tau_sh*(b)).^(1-alpha_sh));
else
    s_direct = ( (alpha_sh.*(l_0.^((sigma_sh-1)/sigma_sh))) + ((1-alpha_sh).*((tau_sh*(b)).^((sigma_sh-1)/sigma_sh))) ).^(sigma_sh/(sigma_sh-1));
end
 

p_direct=zeros(n,num_out);
p_direct_level=zeros(n,num_out);
for zz=1:num_out
    p_direct(:,zz) = ( (theta_sh./alpha_sh).*l_0.*s_direct.*((l_0./s_direct).^(1/sigma_sh)) ) + (psi_2(zz).*(NN_sh*s_direct)) + ([zscore(X) pop]*beta_out(((zz-1)*k)+zz:zz*(k+1))) + (X_d*zet_out((1+((zz*ndep)-ndep):zz*ndep)));
    p_direct_level(:,zz) =((std(p(:,zz))*p_direct(:,zz)) + mean(p(:,zz)));
end

Delta_p_direct=zeros(1,num_out);
for zz=1:num_out
    Delta_p_direct(1,zz) = ((median(p_direct_level(:,zz)) - median(p_base_level(:,zz)))./median(p_base_level(:,zz)));
end

p_direct_med=zeros(1,num_out);
for zz=1:num_out
    p_direct_med(1,zz) = (1+Delta_p_direct(1,zz)).*median(p(:,zz));
end

disp('Observed median prosperity')
disp(median(p))
disp('Counterfactual direct effect median prosperity')
disp(p_direct_med)


















