function [ exp_cond_yy ] = nonparametric_conditional( yy, xx, biweight_kernel, truncation_normal, pphi, nobin )
% Kernel estimator (normal kernel or biweight kernel with fixed bandwidth=1/sqrt(n)).
% It produces the consitional expectation of yy conditional on xx.

global n

number_dependent_variables=length(yy(1,:));
number_independent_variables=length(xx(1,:));

if number_independent_variables==1      % just to be sure we have the right phi in the one variable case
    pphi=1;
end

                    % Ichimura part: construction of the independent variable with the phis as a weight
indep = xx*pphi;

%indep=zeros(n,1);   
%for i=1:number_independent_variables
%    indep=pphi(i)*xx(:,i) + indep;
%end

h=3.5*std(indep)*n^(-1/3);         % bin size
indep_low=min(indep);
indep_high=max(indep);

 
if nobin==1             % if we do not use bins but directly the grid of observations
    grid=indep;
    nbin=n;
else
    nbin=round((indep_high-indep_low)./h)+1;   % number of bins (rounded above)
    
    % create grid
    grid=zeros(nbin,1);
    for bin=1:nbin;
        grid(bin)=indep_low+((bin-1.0)/(nbin-1.0))*(indep_high-indep_low);
    end
end

cond_exp=zeros(nbin,number_dependent_variables);
for dd=1:number_dependent_variables
    k=zeros(nbin,n);    % kernel
    ss=zeros(nbin,n);   % distance between observation and grid point
    aux=zeros(nbin,n);
    
    for bin=1:nbin
        % calculate kernel for each observation given a particular grid point
        for i=1:n
            ss(bin,i)=(grid(bin)-indep(i))/h;
            if ss(bin,i)==0     % exclude one
                k(bin,i)=0;
            else
                % biweight kernel
                if biweight_kernel==1
                    if abs(ss(bin,i))<=1    % if distance from the grid point is too high, the weight on that observation is zero.
                        k(bin,i)=(15/16)*(ss(bin,i)^2-1)^2;
                    end
                    % normal kernel
                else
                    if abs(ss(bin,i))<truncation_normal
                    k(bin,i)=exp(-(ss(bin,i)^2)/2)/(2*pi);
                    end
                end
            end
        end
        denom=sum(k(bin,:));    % denominator
        
        for ii=1:n
            aux(bin,ii)=yy(ii,dd)*k(bin,ii)/denom;
        end
        cond_exp(bin,dd)=sum(aux(bin,:));   % sum over all the observations to get the cond exp in that grid point
    end
    
end

%means = nanmean(cond_exp);
%for dd=1:number_dependent_variables
%    for bin=1:nbin
%        if isnan(cond_exp(bin,dd))==1
%            cond_exp(bin,dd)=means(1,dd);
%        end
%    end
%end
            

% now we need to get the conditional expectations for each observation
if nobin==1
    exp_cond_yy = cond_exp;
else
    exp_cond_yy = zeros(n,number_dependent_variables);
    for ii=1:n
        differ=abs(indep(ii)-grid);
        [diff ind]=min(differ);
        exp_cond_yy(ii,:) = cond_exp(ind);
    end
end
end

