%Estimation of State Capacity Spillovers: NORMATIVE EXERCISE
%Acemoglu, Garcia-Jimeno, Robinson

clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Import data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Open data file State_Capacity_Network_Work.mat
load State_Capacity_Network_Work.mat

%%
%OPTIONS

%1: Choose Outcome variable:
 %out = 3;          %1 = Secondary enrollment 92-02
                    %2 = Services Coverage 1998
                    %3 = Not in poverty 2005
                    %4 = Life Quality Index
                    %5 = Primary enrollment 92-02 (placebo)
                    %6 = Vaccination coverage 1998 (placebo)    
                                       
%2: Choose Measure of State Capacity:
 a_or_b = 0;        %0 = State municipality agencies 
                    %1 = State municipality bureaucracies
 
%3: Choose if control for national-level bureaucracy 
 nat_bur=0;         %0 = no 
                    %1 = yes

%4: Choose if control for rivers density and land shares
 river_land=0;      %0=no
                    %1=yes

%5: Choose if control for distance to current road
current_road=1;     %0=no
                    %1=yes
                    
%6: Choose assumption on Network Links
 ass_network_structure=0;       %0 = Network link is neighbors           
                                %1 = Network link is neighbors and neighbors of neighbors 

%7: Choose order of instruments: 
 order=2;           %2 = Instruments are Nc and N2c
                    %3 = Instruments are N2c and N3c 

%8: Choose the subset of Instruments to use:
 instruments=0;     %0 = Royal roads and Colonial Agencies
                    %1 = Royal roads only
                    %2 = Colonial Agencies only

%9: Choose if exclude violent municipalities
excl_viol=0;        %0 = Do not exclude
                    %1 = Exclude from sample
                    %2 = Exclude from network

%10: Choose how to treat population
pop_treat=0;        %0=Instrument for Population with Historical Population
                    %1=Control for Population                    
%%

 %Choose the endogenous and exogenous variables for the model
 Yg_input = [Yg(:,2) ((Yg(:,4)+Yg(:,5)+Yg(:,6))/3) Yg(:,9) Yg(:,10)];    
         
     %Choose if state capacity is agencies or bureaucracies    
     if a_or_b ==0
         S_input = S(:,6)+S(:,7)+S(:,8)+S(:,11)+S(:,12)+S(:,13)+S(:,14)+S(:,15)+S(:,16)+S(:,18)+S(:,19);                 %S = Contemporary State
     else
         S_input = S(:,1);
     end
         
     C_input = [C(:,1) C(:,3)];                                               %C = Colonial State
     R_input = R;                                                             %R = Distance to royal roads
     
     %Choose controls
     if nat_bur==0 && river_land==0 && current_road==0                        %X = Controls
         X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5)];
        else if nat_bur==0 && river_land==0 && current_road==1
                X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1)];               
            else if nat_bur==0 && river_land==1 && current_road==0
                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) G(:,6:8) L(:,1:7) L(:,12:15)];
                else if nat_bur==0 && river_land==1 && current_road==1
                        X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) G(:,6:8) L(:,1:7) L(:,12:15)];
                    else if nat_bur==1 && river_land==0 && current_road==0
                            X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) log(1+S(:,2))];
                        else if nat_bur==1 && river_land==0 && current_road==1
                                X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) log(1+S(:,2))];
                            else if nat_bur==1 && river_land==1 && current_road==0
                                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) G(:,6:8) L(:,1:7) L(:,12:15) log(1+S(:,2))];
                                else 
                                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) G(:,6:8) L(:,1:7) L(:,12:15) log(1+S(:,2))];
                                end
                            end
                        end
                    end
                end
            end
     end        

     X_input = [X_input Ev(:,1) zscore(NS) H(:,8)./1000 H(:,9)];              %To add 1843 population, missing 1843 pop dummy, Historical electoral variability and Network Statistics
     
     size_X_input = size(X_input,2);                                          %Number of covariates
     size_V = size(Ev(:,1),2) + size(NS,2);                                   %Number of covariates for unobserved national weights
     
     A_input = Adj;                                                           %A = Adjacency matrix
     Dd_input = Ddist;                                                        %Original Distances matrix
     Drr_input = Drankrange;                                                  %Original Range of elevation ranks along path matrix
     Drv_input = Drankvar;                                                    %Original Variance of elevation ranks along path matrix
     Yv_input = Yv(:,16);                                                     %Original Per capita attacks   
     
     
     
     
     %Clean data from missing values
     [Yg_clean S_clean C_clean X_clean R_clean Yv_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean] = clean_missings(Yg_input, S_input, C_input, X_input, R_input, Yv_input, A_input, Dd_input, Drr_input, Drv_input, DaneCodes);

 clear Adj C DD Dc Ddist Drankrange Drankvar G H L R S Xs Yg Yv Ygini Ev DaneCodes NS
 clear Yg_input S_input C_input X_input R_input Yv_input Ev_input A_input Dd_input Drr_input Drv_input


 global p s c r
 p = Yg_clean;                          %p=vectors of prosperity

 s= log(1+S_clean(:,1));                %l=vector of LOCAL state capacity  
 c = (C_clean);                         %c=vector of colonial state presence
 r = zscore(R_clean);                   %r=vector of distance to royal roads
 
 global D E
 D= Dd_clean;                           %D=distance matrix
 E= Drv_clean;                          %E=variance elevation matrix

 global X X_d pop hpop V

 %Choose set of controls
   
    X = X_clean(:,28:end-6);                                                   %matrix of controls
    
    V = [ones(size(p,1),1) zscore(X_clean(:,end-5)) X_clean(:,end-4:end-2)];   %matrix of covariates for the unobserved national weights
    
    hpop = X_clean(:,end-1:end);                                               %matrix of 1843 population and missing 1843 population dummy
    X_d = X_clean(:,2:27);                                                     %matrix of department dummies and department capital dummy
    pop = X_clean(:,1)./1000;                                                  %vector of population
 
 global A
 A=A_clean;                             %A=adjacency matrix (nxn) with 0s on the diagonal

 global n k ndep
 n=size(p,1);                           %number of observations
 k=size(X,2);                           %number of covariates
 ndep=size(X_d,2);                      %number of department dummies
 
 DaneCodes=DaneCodes_clean;             %Municipality codes to keep track
 
 %Create Indicator of violent municipalities (above 90th percentile of attacks)
 v=zeros(n,1);
 pc90 = prctile(Yv_clean,90);
 for j=1:n
     if Yv_clean(j,1)>=pc90
         v(j)=1;
     end
 end
 
 clear Yg_clean S_clean C_clean X_clean R_clean Yv_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean
 
 
 
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Assumptions on the Network Structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if excl_viol==2
    p = p(~v,:);
    s = s(~v,:);
    c = c(~v,:);
    r = r(~v,:);
    X = X(~v,:);
    X_d =X_d(~v,:);
    pop = pop(~v,:);
    hpop = hpop(~v,:);
    
    D0 = D(~v,:)';
    D = (D0(~v,:))';
    
    E0 = E(~v,:)';
    E = (E0(~v,:))';
    
    A0 = A(~v,:)';
    A = (A0(~v,:))';

    clear D0 E0 A0

    n=size(p,1);
end

global NE N N_2 A_2 A_3
F=1./(1+(D.*(1+E)));

%Re-sets the diagonal to zeroes
for i=1:size(F,1)
    F(i,i)=0;
end

Q=(ones(n,n)-A).*(A*A-diag(diag(A*A)));                 %Matrix of Number of Paths to Neighbors of Neighbors who are not own Neighbors
A_2=ismember(Q,1:1:max(max(Q)));                        %Adjacency Matrix of neighbors of neighbors
A_2=+A_2;                                               % from logical to numeric

Q2=(ones(n,n)-A-A_2).*((A*A*A)-diag(diag(A*A*A)));      %Matrix of Number of Paths to Neighbors of Neighbors of Neighbors who are not own Neighbors or own Neighbors of Neighbors
A_3 = ismember(Q2,1:1:max(max(Q2)));                    %Adjacency Matrix of neighbors of neighbors of neighbors
A_3=+A_3;                                               % from logical to numeric
clear Q Q2


if ass_network_structure==1 
    A=(A+A_2);                                          %Adjacency matrix when Network link is defined by assumption b
    A_2=A_3;                                            %Adjacency matrix of neighbors of neighbors when Network link is defined by assumption b
end

%Network matrix of adjacent neighbors
NE = F.*A; 
V(:,3:4)=NE*V(:,3:4);
if order==2
    N=F.*A;                                                 %Network matrix of neighbors
    N_2=F.*A_2;                                             %Network matrix of neighbors of neighbors
else if order==3
        N=F.*A_2;                                           %Network matrix of neighbors of neighbors
        N_2=F.*A_3;                                         %Network matrix of neighbors of neighbors of neighbors
    end
end
 
 %%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %SYSTEM GMM
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% %Municipality-Level FOC
% %s = f( Ns , c1 , c2, r, , X , X_d)
%
% %Outcomes Equation
% %y = f(s^2, N*s, X , X_d)
% 
 %%%%%%First stage GMM
 
 assum_f_delta=0;    % if 0 we are using Assumption f(D,E), 1 if we are using assumption f(D,E,delta).
 options=optimset('Display','iter','MaxIter',40000,'MaxFunEvals',40000,'Algorithm','interior-point');
 biweight_kernel=0;                                      %0 for normal kernel, 1 for biweight kernel
 truncation_normal=160;                                  %Truncation for normal kernel: the lower the value, the stronger the truncation
 nobin=1;                                                %0 for using bins , 1 for using directly the grid of observations
 
 num_out= size(p,2);
 num_eq = num_out+1;  %One equation for each outcome plus the Best Response equation
 
 global d_bar e_bar
     number_of_neighbors=sum(A,2);
     d_bar=(1/1)*(number_of_neighbors).^(-1).*sum(A.*D,2);
     e_bar=(1/1)*(number_of_neighbors).^(-1).*sum(A.*E,2);

     %Initialize parameter value for delta
      ddelta_guess = 1*ones(1,2);    
      
 if assum_f_delta==1     % if N(d) depends on delta (the neighbors matrices are assumed to depend on a parameter vector delta)
          
      ddelta=ddelta_guess;
      [ N_d, N_d_2, F_d] = neighbors_matrix( ddelta, order );  % call a function that generates the neighbors and neighbors of neighbors matrices which depend on delta
      NN=N_d;
      NN_2=N_d_2;    
     
      Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (NN*r).^3 (NN*log(1+c)).^4 (NN*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop zscore(d_bar) zscore(e_bar)];
      Z_br_w=[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2) zscore(d_bar) zscore(e_bar)];
                 
      lb = [  0   0 -Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep)    0    0]';
      ub = [Inf Inf  Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep)  Inf  Inf]';    
 else
     NN=N;
     NN_2=N_2;
          
     Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (NN*r).^3 (NN*log(1+c)).^4 (NN*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop];
     Z_br_w=[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2)];
                 
       
     lb = [  0   0 -Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep)]';
     ub = [Inf Inf  Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep)]';  
 end
 
 num_mom = (num_out*size(Z_out_w,2))+size(Z_br_w,2); %total number of moments
 
 %Instruments for the Weighting  Matrix
    Zw = zeros(num_eq,num_mom,n);
    for obs_n=1:n
        Zw(:,:,obs_n) = [kron(eye(num_out),Z_out_w(obs_n,:)) zeros(num_out,size(Z_br_w,2)); zeros(1,(num_out*size(Z_out_w,2))) Z_br_w(obs_n,:)];
    end  
  
    ZOZ=zeros(num_mom,num_mom);
 
    for i=1:n
        ZOZ = Zw(:,:,i)'*Zw(:,:,i) + ZOZ;
    end

    %Initialize parameter values
      
      theta_guess = 1;      
      psi_1_guess = 1;
      psi_2_guess = 0.01*ones(1,num_out);
      beta_out_guess = 0*ones(1,num_out*(k+1));
      zet_out_guess = 0*ones(1,num_out*ndep);
      phi_guess = 0.01*ones(1,3);
      beta_br_guess = 0.01*ones(1,k+1);
      zet_br_guess = 0.01*ones(1,ndep);

      if assum_f_delta==1
          guess_gmm = [theta_guess psi_1_guess psi_2_guess beta_out_guess zet_out_guess phi_guess beta_br_guess zet_br_guess ddelta_guess];   
          tic
            [param_gmm_first, fval_first] = fmincon(@(param_gmm) sys_gmm_objective_function(param_gmm, assum_f_delta, truncation_normal, nobin, biweight_kernel, order, num_out, ZOZ ), guess_gmm, [], [], [], [], lb, ub, [], options);
          toc
      
      else
          guess_gmm = [theta_guess psi_1_guess psi_2_guess beta_out_guess zet_out_guess phi_guess beta_br_guess zet_br_guess];
          tic
            [param_gmm_first, fval_first] = fmincon(@(param_gmm) sys_gmm_objective_function(param_gmm, assum_f_delta, truncation_normal, nobin, biweight_kernel, order, num_out, ZOZ ), guess_gmm, [], [], [], [], lb, ub, [], options);
          toc
      end
  
 
 %%%%%%Second stage (efficient) System GMM
      
      theta = param_gmm_first(1);
      psi_1 = param_gmm_first(2);
      psi_2 = param_gmm_first(3:3+num_out-1);
      beta_out = param_gmm_first(3+num_out:3+num_out+(num_out*(k+1))-1)';
      zet_out = param_gmm_first(3+num_out+(num_out*(k+1)):3+num_out+(num_out*(k+1))+(num_out*ndep)-1)';
      phi = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep):3+num_out+(num_out*(k+1))+(num_out*ndep)+3-1)';
      beta_br = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep)+3:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1-1)';
      zet_br = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1)';     
      
if assum_f_delta==1                                            % if N(d) depends on delta (the neighbors matrices are assumed to depend on a parameter vector delta)      
      ddelta = param_gmm_first(end-1:end);   
      [ N_d, N_d_2, F_d] = neighbors_matrix( ddelta, order );  % call a function that generates the neighbors and neighbors of neighbors matrices which depend on delta
      NN=N_d;
      NN_2=N_d_2;
      
      Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop zscore(d_bar) zscore(e_bar)];
      Z_br_w=[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2) zscore(d_bar) zscore(e_bar)];
      
else
     NN=N;
     NN_2=N_2;
     
     Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop];
     Z_br_w=[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2)];
end      

%Non parametric conditional expectation
    pphi=[phi; beta_br];
    xx=[log(1+c) r zscore(X) log(pop)];    
    exp_cond_g= nonparametric_conditional( s - ((psi_1/theta)*(NN*s)) - (X_d*zet_br), xx, biweight_kernel, truncation_normal, pphi, nobin );  % It gives the conditional expectation of s+ppsi_gmm*N*s-X*bbeta_gmm conditional on c and lambda for each bin

    %Residuals:
    Qt = zeros(n,num_out);
    for i=1:num_out
        Qt(:,i) = zscore(p(:,i)) - (theta*(s.^2)) - (psi_2(i)*(NN*s)) - ([zscore(X) pop]*beta_out(((i-1)*k)+i:i*(k+1))) - (X_d*zet_out(1+((i*ndep)-ndep):i*ndep)); 
    end
        q_br = s - ((psi_1/theta)*(NN*s)) - (X_d*zet_br) - exp_cond_g;
        Qt = [Qt  q_br];
        Q=Qt';
        
   
    %Construction of spatially correlated weighting matrix    
    OM0=zeros(num_eq,num_eq,n);
    for obs_n=1:n
        OM0(:,:,obs_n) =Q(:,obs_n)*Q(:,obs_n)';
    end    
    
    OM1=zeros(num_eq,num_eq,n);
    for i=1:n
        neigh_i=0;
        VUi=zeros(num_eq,num_eq,n);
        for j=1:n
            if A(i,j)==1
                neigh_i=neigh_i+1;
                VUi(:,:,j)=Q(:,i)*Q(:,j)';
            end
        end
        OM1(:,:,i)=(1/neigh_i)*sum(VUi,3);
    end
    
    OM2=zeros(num_eq,num_eq,n);
    for i=1:n
        neigh2_i=0;
        VU2i=zeros(num_eq,num_eq,n);
        for j=1:n
            if A_2(i,j)==1
                neigh2_i=neigh2_i+1;
                VU2i(:,:,j)=Q(:,i)*Q(:,j)';
            end
        end
        OM2(:,:,i)=(1/neigh2_i)*sum(VU2i,3);
    end
  
    %Instruments for the Weighting  Matrix
        Zw = zeros(num_eq,num_mom,n);
        for obs_n=1:n
            Zw(:,:,obs_n) = [kron(eye(num_out),Z_out_w(obs_n,:)) zeros(num_out,size(Z_br_w,2)); zeros(1,(num_out*size(Z_out_w,2))) Z_br_w(obs_n,:)];
        end  
        
    ZOZ=zeros(num_mom,num_mom);
    for i=1:n
        ZOZ = Zw(:,:,i)'*(OM0(:,:,i) +((2/23)*(OM1(:,:,i) + OM1(:,:,i)')) + ((1/23)*(OM2(:,:,i) + OM2(:,:,i)')) )*Zw(:,:,i) + ZOZ;
    end
    
 guess_gmm = param_gmm_first;
 options=optimset('Display','iter','MaxIter',200000,'MaxFunEvals',200000,'Algorithm','interior-point');
         tic
         [param_gmm_final, fval_final] = fmincon(@(param_gmm) sys_gmm_objective_function(param_gmm, assum_f_delta, truncation_normal, nobin, biweight_kernel, order, num_out, ZOZ ), guess_gmm, [], [], [], [], lb, ub, [], options);
         toc
 
         
      theta = param_gmm_final(1);
      psi_1 = param_gmm_final(2);
      psi_2 = param_gmm_final(3:3+num_out-1);
      beta_out = param_gmm_final(3+num_out:3+num_out+(num_out*(k+1))-1)';
      zet_out = param_gmm_final(3+num_out+(num_out*(k+1)):3+num_out+(num_out*(k+1))+(num_out*ndep)-1)';
      phi = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep):3+num_out+(num_out*(k+1))+(num_out*ndep)+3-1)';
      beta_br = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep)+3:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1-1)';
      zet_br = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1)';
      
      
 if assum_f_delta==1                                            % if N(d) depends on delta (the neighbors matrices are assumed to depend on a parameter vector delta)      
      ddelta = param_gmm_final(end-1:end); 
      [ N_d, N_d_2, F_d] = neighbors_matrix( ddelta, order );  % call a function that generates the neighbors and neighbors of neighbors matrices which depend on delta
      NN=N_d;
      NN_2=N_d_2; 
 else
     NN=N;
     NN_2=N_2;
 end      

 
 
disp('parameters estimates for System GMM')
disp('theta')
disp(theta)
disp('psi1')
disp(psi_1)
disp('psi_2')
disp(psi_2)
disp('phi')
disp(phi')
disp('Average kappa')

if assum_f_delta==1
    disp('delta')
    disp(ddelta')
end

if assum_f_delta==1       
    [ N_d, N_d_2, F_d] = neighbors_matrix( ddelta, order );
    NN=N_d;
    NN_2=N_d_2;
else
    NN=N;
    NN_2=N_2;
end      

%Kappa
    pphi=[phi; beta_br];
    xx=[log(1+c) r zscore(X) log(pop)];    
    kappas = theta*nonparametric_conditional( s - ((psi_1/theta)*(NN*s)) - (X_d*zet_br), xx, biweight_kernel, truncation_normal, pphi, nobin );
    
disp(mean(kappas))

%%
%Optimal allocation

Experiment = log(1);
pop_weight = (pop)./sum(pop);
w_tilda =1./pop_weight;

M = eye(n)/(eye(n) - ((psi_1/theta)*NN));

g=zeros(n,1);
for i=1:n
    
    g(i) = 2*ones(1,n)*(diag(M(i,:)))*M*(kappas);
end

h=zeros(n,1);
for i=1:n
    
    h(i) = mean(psi_2)*ones(1,n)*NN*M(i,:)';
end

Q = zeros(n,n);
for i=1:n
    
    Q(i,:) = 2*theta*ones(1,n)*diag(M(i,:))*M;
end


B = [0 ones(1,n); -w_tilda Q];

optimal = B\[Experiment; -g - h];

lambda = optimal(1);
e = optimal(2:end);

disp('Shadow price')
disp(lambda)

%Regression of optimal allocation on centrality


%Betweenness
XX = [V(:,3) zscore(X) (zscore(X).^2) log(pop) (log(pop).^2) X_d ones(n,1)];

bbetas = (XX'*XX)\(XX'*e);

disp('coefficient on betweenness')
disp(bbetas(1))

disp('coefficient on population')
disp(bbetas(14))

    % standard errors
    resid=e-XX*bbetas;
    % construction of the variance of the coefficients estimators
    inside_part=0;
    for i=1:n
        inside_part=(resid(i))^2*XX(i,:)'*XX(i,:) + inside_part;               
    end
    var_bbetas=((XX'*XX)\(inside_part)/(XX'*XX));
    std_bbetas=sqrt(diag(var_bbetas));                                            % standard errors
    disp('standard error for betweenness')
    disp(std_bbetas(1))
    disp('standard error for population')
    disp(std_bbetas(14))
    
    % spatial correlation-corrected standard errors
    OMEG= resid*resid';
    VU = (OMEG.*eye(n)) + ((2/3)*((OMEG.*NN)+(OMEG.*NN)')) + ((1/3)*((OMEG.*NN_2)+(OMEG.*NN_2)'));
    var_bbetas_spatial = (XX'*XX)\XX'*VU*XX/(XX'*XX);
    std_bbetas_spatial = sqrt(diag(var_bbetas_spatial));
    disp('spatial standard error on betweenness')
    disp(std_bbetas_spatial(1))
    disp('spatial standard error for population')
    disp(std_bbetas_spatial(14))
    
    R_sq = 1 - ((sum(resid.*resid)/sum((((e) - (sum(e)/n)).^2))));
    disp('R squared')
    disp(R_sq)    
    
%Bonacich
XX = [V(:,4) zscore(X) (zscore(X).^2) log(pop) (log(pop).^2) X_d ones(n,1)];

bbetas = (XX'*XX)\(XX'*e);

disp('coefficient on bonacich')
disp(bbetas(1))

disp('coefficient on population')
disp(bbetas(14))

    % standard errors
    resid=e-XX*bbetas;
    % construction of the variance of the coefficients estimators
    inside_part=0;
    for i=1:n
        inside_part=(resid(i))^2*XX(i,:)'*XX(i,:) + inside_part;               
    end
    var_bbetas=((XX'*XX)\(inside_part)/(XX'*XX));
    std_bbetas=sqrt(diag(var_bbetas));                                            % standard errors
    disp('standard error on bonacich')
    disp(std_bbetas(1))
    disp('standard error for population')
    disp(std_bbetas(14))
    
    % spatial correlation-corrected standard errors
    OMEG= resid*resid';
    VU = (OMEG.*eye(n)) + ((2/3)*((OMEG.*NN)+(OMEG.*NN)')) + ((1/3)*((OMEG.*NN_2)+(OMEG.*NN_2)'));
    var_bbetas_spatial = (XX'*XX)\XX'*VU*XX/(XX'*XX);
    std_bbetas_spatial = sqrt(diag(var_bbetas_spatial));
    disp('spatial standard errors on bonacich')
    disp(std_bbetas_spatial(1))
    disp('spatial standard error for population')
    disp(std_bbetas_spatial(14))

    
    R_sq = 1 - ((sum(resid.*resid)/sum((((e) - (sum(e)/n)).^2))));
    disp('R squared')
    disp(R_sq)   
 
%Clustering    
    XX = [V(:,5) zscore(X) (zscore(X).^2) log(pop) (log(pop).^2) X_d ones(n,1)];

    bbetas = (XX'*XX)\(XX'*e);

    disp('coefficient on clustering')
    disp(bbetas(1))

    disp('coefficient on population')
    disp(bbetas(14))

    % standard errors
    resid=e-XX*bbetas;
    % construction of the variance of the coefficients estimators
    inside_part=0;
    for i=1:n
        inside_part=(resid(i))^2*XX(i,:)'*XX(i,:) + inside_part;               
    end
    var_bbetas=((XX'*XX)\(inside_part)/(XX'*XX));
    std_bbetas=sqrt(diag(var_bbetas));                                            % standard errors
    disp('standard error on clustering')
    disp(std_bbetas(1))
    disp('standard error for population')
    disp(std_bbetas(14))
    
    % spatial correlation-corrected standard errors
    OMEG= resid*resid';
    VU = (OMEG.*eye(n)) + ((2/3)*((OMEG.*NN)+(OMEG.*NN)')) + ((1/3)*((OMEG.*NN_2)+(OMEG.*NN_2)'));
    var_bbetas_spatial = (XX'*XX)\XX'*VU*XX/(XX'*XX);
    std_bbetas_spatial = sqrt(diag(var_bbetas_spatial));
    disp('spatial standard errors on clustering')
    disp(std_bbetas_spatial(1))
    disp('spatial standard error for population')
    disp(std_bbetas_spatial(14))

    
    R_sq = 1 - ((sum(resid.*resid)/sum((((e) - (sum(e)/n)).^2))));
    disp('R squared')
    disp(R_sq)   
    
%Betweenness, Bonacich, and Clustering
XX = [V(:,3:5) zscore(X) (zscore(X).^2) log(pop) (log(pop).^2) X_d ones(n,1)];

bbetas = (XX'*XX)\(XX'*e);

disp('coefficients on betweenness, bonacich, and clustering')
disp(bbetas(1:3))

disp('coefficient on population')
disp(bbetas(16))

    % standard errors
    resid=e-XX*bbetas;
    % construction of the variance of the coefficients estimators
    inside_part=0;
    for i=1:n
        inside_part=(resid(i))^2*XX(i,:)'*XX(i,:) + inside_part;               
    end
    var_bbetas=((XX'*XX)\(inside_part)/(XX'*XX));
    std_bbetas=sqrt(diag(var_bbetas));                                            % standard errors
    disp('standard errors on betweenness, bonacich, and clustering')
    disp(std_bbetas(1:3))
    disp('standard error for population')
    disp(std_bbetas(16))
    
    % spatial correlation-corrected standard errors
    OMEG= resid*resid';
    VU = (OMEG.*eye(n)) + ((2/3)*((OMEG.*NN)+(OMEG.*NN)')) + ((1/3)*((OMEG.*NN_2)+(OMEG.*NN_2)'));
    var_bbetas_spatial = (XX'*XX)\XX'*VU*XX/(XX'*XX);
    std_bbetas_spatial = sqrt(diag(var_bbetas_spatial));
    disp('spatial standard errors on betweenness, bonacich, and clustering')
    disp(std_bbetas_spatial(1:3))
    disp('spatial standard error for population')
    disp(std_bbetas_spatial(16))


    R_sq = 1 - ((sum(resid.*resid)/sum((((e) - (sum(e)/n)).^2))));
    disp('R squared')
    disp(R_sq)   

 %predicted e to take to simul_gmm.m
 e_predict = XX*bbetas;
    
%%
%Welfare Gains
e_fix=std(s)*(zscore(e));
s_exper = M*( ((1/theta)*kappas) + (e_fix) );

p_exper=zeros(n,num_out);
for i=1:num_out
 
    p_exper(:,i) = kappas.*s_exper + psi_1*(s_exper.*(NN*s_exper)) + psi_2(i)*NN*s_exper + zscore(X)*beta_out(1+((i-1)*k):i*k) + X_d*zet_out(1+((i-1)*ndep):i*ndep); 
    
    p_exper(:,i) = (p_exper(:,i)*std(p(:,i))) + mean(p(:,i)); 
end

s_base = s;

p_base=zeros(n,num_out);
for i=1:num_out

    p_base(:,i) = kappas.*s + psi_1*(s.*(NN*s)) + psi_2(i)*NN*s + zscore(X)*beta_out(1+((i-1)*k):i*k) + X_d*zet_out(1+((i-1)*ndep):i*ndep); 
    
    p_base(:,i) = (p_base(:,i)*std(p(:,i))) + mean(p(:,i)); 
end

Delta = p.*((p_exper - p_base)./p_base);
mean(Delta)
median(Delta)


welfare_gain_exper = (p_exper-p_base);

disp('Raw Average Welfare Gains from Optimal Allocation')
disp(mean(welfare_gain_exper))

disp('Median Welfare Gains from Optimal Allocation')
disp(median(welfare_gain_exper))












