%Estimation of State Capacity Spillovers: Specification Test
%Acemoglu, Garcia-Jimeno, Robinson

clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Import data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Open data file State_Capacity_Network_Work.mat
load State_Capacity_Network_Work.mat

%%
%OPTIONS

%1: Choose Outcome variable:
 %out = 3;          %1 = Secondary enrollment 92-02
                    %2 = Services Coverage 1998
                    %3 = Not in poverty 2005
                    %4 = Life Quality Index
                    %5 = Primary enrollment 92-02 (placebo)
                    %6 = Vaccination coverage 1998 (placebo)    
                                       
%2: Choose Measure of State Capacity:
 a_or_b = 0;        %0 = State municipality agencies 
                    %1 = State municipality bureaucracies
 
%3: Choose if control for national-level bureaucracy 
 nat_bur=0;         %0 = no 
                    %1 = yes

%4: Choose if control for rivers density and land shares
 river_land=0;      %0=no
                    %1=yes

%5: Choose if control for distance to current road
current_road=1;     %0=no
                    %1=yes
                    
%6: Choose assumption on Network Links
 ass_network_structure=0;       %0 = Network link is neighbors           
                                %1 = Network link is neighbors and neighbors of neighbors 

%7: Choose order of instruments: 
 order=2;           %2 = Instruments are Nc and N2c
                    %3 = Instruments are N2c and N3c 

%8: Choose the subset of Instruments to use:
 instruments=0;     %0 = Royal roads and Colonial Agencies
                    %1 = Royal roads only
                    %2 = Colonial Agencies only

%9: Choose if exclude violent municipalities
excl_viol=0;        %0 = Do not exclude
                    %1 = Exclude from sample
                    %2 = Exclude from network

%10: Choose how to treat population
pop_treat=0;        %0=Instrument for Population with Historical Population
                    %1=Control for Population                    
%%

%Choose the endogenous and exogenous variables for the model
 Yg_input = [Yg(:,2) ((Yg(:,4)+Yg(:,5)+Yg(:,6))/3) Yg(:,9) Yg(:,10)]; 
         

         
     %Choose if state capacity is agencies or bureaucracies    
     if a_or_b ==0
         S_input = S(:,6)+S(:,7)+S(:,8)+S(:,11)+S(:,12)+S(:,13)+S(:,14)+S(:,15)+S(:,16)+S(:,18)+S(:,19);                 %S = Contemporary State
     else
         S_input = S(:,1);
     end
         
     C_input = [C(:,1) C(:,3)];                                               %C = Colonial State
     R_input = R;                                                             %R = Distance to royal roads
     
     %Choose controls
     if nat_bur==0 && river_land==0 && current_road==0                        %X = Controls
         X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5)];
        else if nat_bur==0 && river_land==0 && current_road==1
                X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1)];               
            else if nat_bur==0 && river_land==1 && current_road==0
                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) G(:,6:8) L(:,1:7) L(:,12:15)];
                else if nat_bur==0 && river_land==1 && current_road==1
                        X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) G(:,6:8) L(:,1:7) L(:,12:15)];
                    else if nat_bur==1 && river_land==0 && current_road==0
                            X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) log(1+S(:,2))];
                        else if nat_bur==1 && river_land==0 && current_road==1
                                X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) log(1+S(:,2))];
                            else if nat_bur==1 && river_land==1 && current_road==0
                                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) G(:,6:8) L(:,1:7) L(:,12:15) log(1+S(:,2))];
                                else 
                                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) G(:,6:8) L(:,1:7) L(:,12:15) log(1+S(:,2))];
                                end
                            end
                        end
                    end
                end
            end
     end        

                                                        
     X_input = [X_input Ev(:,1) NS H(:,8)./1000 H(:,9)];                      %To add 1843 population, missing 1843 pop dummy, Historical electoral variability and Network Statistics
     
     size_X_input = size(X_input,2);                                          %Number of covariates
     size_V = size(Ev(:,1),2) + size(NS,2);                                   %Number of covariates for unobserved national weights
     
     A_input = Adj;                                                           %A = Adjacency matrix
     Dd_input = Ddist;                                                        %Original Distances matrix
     Drr_input = Drankrange;                                                  %Original Range of elevation ranks along path matrix
     Drv_input = Drankvar;                                                    %Original Variance of elevation ranks along path matrix
     Yv_input = Yv(:,16);                                                     %Original Per capita attacks   
     
     
     %Clean data from missing values
     [Yg_clean S_clean C_clean X_clean R_clean Yv_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean] = clean_missings(Yg_input, S_input, C_input, X_input, R_input, Yv_input, A_input, Dd_input, Drr_input, Drv_input, DaneCodes);

 clear Adj C DD Dc Ddist Drankrange Drankvar G H L R S Xs Yg Yv Ygini Ev DaneCodes NS
 clear Yg_input S_input C_input X_input R_input Yv_input Ev_input A_input Dd_input Drr_input Drv_input


 global p l b c r
 p = Yg_clean;                          %p=vectors of prosperity

 s= log(1+S_clean(:,1));                %l=vector of LOCAL state capacity  
 
 c = (C_clean);                         %c=vector of colonial state presence
 r = zscore(R_clean);                   %r=vector of distance to royal roads
 
 global D E
 D= Dd_clean;                           %D=distance matrix
 E= Drv_clean;                          %E=variance elevation matrix

 global X X_d pop hpop V

 %Choose set of controls
   
    X = X_clean(:,28:end-5);                                                %matrix of controls
    
    V = [ones(size(p,1),1) X_clean(:,end-4:end-2)];                         %matrix of covariates for the unobserved national weights
    
    hpop = X_clean(:,end-1:end);                                            %matrix of 1843 population and missing 1843 population dummy
    X_d = X_clean(:,2:27);                                                  %matrix of department dummies and department capital dummy
    pop = X_clean(:,1)./1000;                                               %vector of population
 
 global A
 A=A_clean;                             %A=adjacency matrix (nxn) with 0s on the diagonal

 global n k ndep
 n=size(p,1);                           %number of observations
 k=size(X,2);                           %number of covariates
 ndep=size(X_d,2);                      %number of department dummies
 
 DaneCodes=DaneCodes_clean;             %Municipality codes to keep track
 
 %Create Indicator of violent municipalities (above 90th percentile of attacks)
 v=zeros(n,1);
 pc90 = prctile(Yv_clean,90);
 for j=1:n
     if Yv_clean(j,1)>=pc90
         v(j)=1;
     end
 end
 
 clear Yg_clean S_clean C_clean X_clean R_clean Yv_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean
 
 
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Assumptions on the Network Structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if excl_viol==2
    p = p(~v,:);
    s = s(~v,:);
    c = c(~v,:);
    r = r(~v,:);
    X = X(~v,:);
    X_d =X_d(~v,:);
    pop = pop(~v,:);
    hpop = hpop(~v,:);
    V = V(~v,:); 
    
    D0 = D(~v,:)';
    D = (D0(~v,:))';
    
    E0 = E(~v,:)';
    E = (E0(~v,:))';
    
    A0 = A(~v,:)';
    A = (A0(~v,:))';

    clear D0 E0 A0

    n=size(p,1);
end

global NE N N_2 A_2 A_3
F=1./(1+(D.*(1+E)));

%Re-sets the diagonal to zeroes
for i=1:size(F,1)
    F(i,i)=0;
end

Q=(ones(n,n)-A).*(A*A-diag(diag(A*A)));                 %Matrix of Number of Paths to Neighbors of Neighbors who are not own Neighbors
A_2=ismember(Q,1:1:max(max(Q)));                        %Adjacency Matrix of neighbors of neighbors
A_2=+A_2;                                               % from logical to numeric

Q2=(ones(n,n)-A-A_2).*((A*A*A)-diag(diag(A*A*A)));      %Matrix of Number of Paths to Neighbors of Neighbors of Neighbors who are not own Neighbors or own Neighbors of Neighbors
A_3 = ismember(Q2,1:1:max(max(Q2)));                    %Adjacency Matrix of neighbors of neighbors of neighbors
A_3=+A_3;                                               % from logical to numeric
clear Q Q2


if ass_network_structure==1 
    A=(A+A_2);                                          %Adjacency matrix when Network link is defined by assumption b
    A_2=A_3;                                            %Adjacency matrix of neighbors of neighbors when Network link is defined by assumption b
end

%Network matrix of adjacent neighbors
NE = F.*A; 

if order==2
    N=F.*A;                                                 %Network matrix of neighbors
    N_2=F.*A_2;                                             %Network matrix of neighbors of neighbors
else if order==3
        N=F.*A_2;                                           %Network matrix of neighbors of neighbors
        N_2=F.*A_3;                                         %Network matrix of neighbors of neighbors of neighbors
    end
end

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Best response Equation 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Linear Model
%s = f( N*s , c1 , c2 , X , X_d)
%N*s = g(N*c , N*r , N_2*c , N_2*r , X , X_d)

endog_var = NE*s;

if pop_treat==0

    if a_or_b<2
        if instruments==0
            Z =     [N*log(1+c) N*r N_2*log(1+c) N_2*r log(1+c) r zscore(X) X_d ones(n,1) log(hpop(:,1)) hpop(:,2)];                                                     %Benchmark
            XX =     [endog_var                         log(1+c) r zscore(X) X_d ones(n,1)                 log(pop)];
            
            ZZZf = [N*log(1+c) N*r N_2*log(1+c) N_2*r log(hpop(:,1)) hpop(:,2)];
            XXXf = [log(1+c) r zscore(X) X_d ones(n,1)];
    
        else if instruments==1
                 Z = [           N*r              N_2*r          r zscore(X) X_d ones(n,1) log(hpop(:,1)) hpop(:,2)];
                XX = [endog_var                                  r zscore(X) X_d ones(n,1)                 log(pop)];
                
                 ZZZf = [ N*r N_2*r log(hpop(:,1)) hpop(:,2)];
                 XXXf = [ r zscore(X) X_d ones(n,1)];
                
            else
                 Z = [N*log(1+c)     N_2*log(1+c)       log(1+c)   zscore(X) X_d ones(n,1) log(hpop(:,1)) hpop(:,2)];
                XX = [endog_var          log(1+c)                  zscore(X) X_d ones(n,1)                 log(pop)];
                
                ZZZf = [N*log(1+c) N_2*log(1+c) log(hpop(:,1)) hpop(:,2)];
                XXXf = [ log(1+c) zscore(X) X_d ones(n,1)];
            end
        end
        
    else if a_or_b>=2
            if instruments==0
                 Z = [N*log(1+c) N*r N_2*log(1+c) N_2*r             log(1+c) r zscore(X) X_d ones(n,1) log(hpop(:,1)) hpop(:,2)];
                XX = [endog_var                                     log(1+c) r zscore(X) X_d ones(n,1)                 log(pop)];
            end
        end
    end

else if pop_treat==1
        
        if a_or_b<2
            if instruments==0
                 Z =     [N*log(1+c) N*r N_2*log(1+c) N_2*r log(1+c) r zscore(X) X_d ones(n,1) log(pop)];                                                     %Benchmark
                XX =     [endog_var                         log(1+c) r zscore(X) X_d ones(n,1) log(pop)];
                
                ZZZr = [log(1+c) r zscore(X) X_d ones(n,1) log(pop)];
                
            else if instruments==1
                     Z = [           N*r              N_2*r          r zscore(X) X_d ones(n,1) log(pop)];
                    XX = [endog_var                                  r zscore(X) X_d ones(n,1) log(pop)];
                    
                    ZZZr = [r zscore(X) X_d ones(n,1) log(pop)];
                else
                     Z = [N*log(1+c)     N_2*log(1+c)       log(1+c)   zscore(X) X_d ones(n,1) log(pop)];
                    XX = [endog_var          log(1+c)                  zscore(X) X_d ones(n,1) log(pop)];
                    
                    ZZZr = [log(1+c) zscore(X) X_d ones(n,1) log(pop)];
                end
            end
            
        else if a_or_b>=2
                if instruments==0
                     Z = [N*log(1+c) N*r N_2*log(1+c) N_2*r             log(1+c) r zscore(X) X_d ones(n,1) log(pop)];
                    XX = [endog_var                                     log(1+c) r zscore(X) X_d ones(n,1) log(pop)];
                end
            end
        end
    end
end      

YY = s;

    if excl_viol==1
        Z = Z(~v,:);
        XX = XX(~v,:);
        YY = YY(~v,:);
        n = size(YY,1);
        
        if pop_treat==0 & a_or_b<2
            ZZZf = ZZZf(~v,:);
            XXXf = XXXf(~v,:);
        else if pop_treat==1 & a_or_b<2
                ZZZr = ZZZr(~v,:);
            end
        end
    end

 %Marginal Effects
    Avg_nij = N(:);
    Avg_nij = mean(Avg_nij(Avg_nij~=0));   


%IV
Q= XX'*Z*(inv(Z'*Z))*Z';
coeff_IV_BR=(Q*XX)\(Q*YY);             % estimation of the coefficients

ppsi1=coeff_IV_BR(1);
phi=coeff_IV_BR(2:4);
bbeta=coeff_IV_BR(5:end);
disp('coefficients estimates best response IV')
disp([coeff_IV_BR(1:6)]')

    % standard errors
    resid=YY-XX*coeff_IV_BR;
    % construction of the variance of the coefficients estimators
    inside_part=0;
    for i=1:n
        inside_part=(resid(i))^2*Z(i,:)'*Z(i,:) + inside_part;               
    end
    var_coeff_IV_BR=((Q*XX)\(XX'*Z*(inv(Z'*Z)))*inside_part*((inv(Z'*Z))*Z'*XX)/(Q*XX));
    std_error_IV_BR=sqrt(diag(var_coeff_IV_BR));                                            % standard errors
    disp('standard errors for Best response IV')
    disp([std_error_IV_BR(1:6)]')

    %Spatial correlation-corrected standard errors
    OMEG= resid*resid';
    VU = (OMEG.*eye(n)) + ((2/3)*((OMEG.*N)+(OMEG.*N)')) + ((1/3)*((OMEG.*N_2)+(OMEG.*N_2)'));
    var_coeff_IV_BR_Spatial = (Q*XX)\Q*VU*Q'/(Q*XX);
    std_error_IV_BR_Spatial = sqrt(diag(var_coeff_IV_BR_Spatial)); 
    disp('spatial standard errors for Best response IV')
    disp([std_error_IV_BR_Spatial(1:6)]')


SPEC(:,1)=resid;

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Outcomes Equation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

%Structural Equation
disp('structural')
    
if pop_treat==0
    XXXf = [zscore(X) X_d]; 
    if instruments==0
        Z = [N*log(1+c) N*r (N*log(1+c)).^2 (N*r).^2 (N*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 N_2*log(1+c) N_2*r (N_2*log(1+c)).^2 (N_2*r).^2 (N_2*log(1+c)).^3 (N_2*r).^3 (N_2*log(1+c)).^4 (N_2*r).^4 zscore(X) X_d hpop];
        
        ZZZf = [N*log(1+c) N*r (N*log(1+c)).^2 (N*r).^2 (N*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 N_2*log(1+c) N_2*r (N_2*log(1+c)).^2 (N_2*r).^2 (N_2*log(1+c)).^3 (N_2*r).^3 (N_2*log(1+c)).^4 (N_2*r).^4 hpop];
    else if instruments==1
            Z = [N*log(1+c(:,2)) N*r (N*log(1+c(:,2))).^2 (N*r).^2 (N*log(1+c(:,2))).^3 (N*r).^3 (N*log(1+c(:,2))).^4 (N*r).^4 N_2*log(1+c(:,2)) N_2*r (N_2*log(1+c(:,2))).^2 (N_2*r).^2  zscore(X) X_d hpop];
            
            ZZZf = [N*log(1+c(:,2)) N*r (N*log(1+c(:,2))).^2 (N*r).^2 (N*log(1+c(:,2))).^3 (N*r).^3 (N*log(1+c(:,2))).^4 (N*r).^4 N_2*log(1+c(:,2)) N_2*r (N_2*log(1+c(:,2))).^2 (N_2*r).^2 hpop];
        else if instruments==2
            Z = [N*log(1+c) (N*log(1+c)).^2 (N*log(1+c)).^3 (N*log(1+c)).^4 N_2*log(1+c) (N_2*log(1+c)).^2 (N_2*log(1+c)).^3 (N_2*log(1+c)).^4 zscore(X) X_d hpop];
            
            ZZZf = [N*log(1+c) (N*log(1+c)).^2 (N*log(1+c)).^3 (N*log(1+c)).^4 N_2*log(1+c) (N_2*log(1+c)).^2 (N_2*log(1+c)).^3 (N_2*log(1+c)).^4 hpop];
            else if instruments==3
                Z = [N*r (N*r).^2 (N*r).^3 (N*r).^4 N_2*r (N_2*r).^2 (N_2*r).^3 (N_2*r).^4 zscore(X) X_d hpop];
        
                ZZZf = [N*r (N*r).^2 (N*r).^3 (N*r).^4 N_2*r (N_2*r).^2 (N_2*r).^3 (N_2*r).^4 hpop];
                else if instruments==4
                        Z = [N*log(1+c(:,1)) (N*log(1+c(:,1))).^2 (N*log(1+c(:,1))).^3 (N*log(1+c(:,1))).^4 N_2*log(1+c(:,1)) (N_2*log(1+c(:,1))).^2 (N_2*log(1+c(:,1))).^3 (N_2*log(1+c(:,1))).^4 zscore(X) X_d hpop];
        
                        ZZZf = [N*log(1+c(:,1)) (N*log(1+c(:,1))).^2 (N*log(1+c(:,1))).^3 (N*log(1+c(:,1))).^4 N_2*log(1+c(:,1)) (N_2*log(1+c(:,1))).^2 (N_2*log(1+c(:,1))).^3 (N_2*log(1+c(:,1))).^4 hpop];
                    else 
                        Z = [N*log(1+c(:,2)) (N*log(1+c(:,2))).^2 (N*log(1+c(:,2))).^3 (N*log(1+c(:,2))).^4 N_2*log(1+c(:,2)) (N_2*log(1+c(:,2))).^2 (N_2*log(1+c(:,2))).^3 (N_2*log(1+c(:,2))).^4 zscore(X) X_d hpop];
        
                        ZZZf = [N*log(1+c(:,2)) (N*log(1+c(:,2))).^2 (N*log(1+c(:,2))).^3 (N*log(1+c(:,2))).^4 N_2*log(1+c(:,2)) (N_2*log(1+c(:,2))).^2 (N_2*log(1+c(:,2))).^3 (N_2*log(1+c(:,2))).^4 hpop];
                    end
                end
            end
        end
    end
 
else if pop_treat==1
        XXXf = [zscore(X) X_d pop]; 
        if instruments==0
            Z = [N*log(1+c) N*r (N*log(1+c)).^2 (N*r).^2 (N*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 N_2*log(1+c) N_2*r (N_2*log(1+c)).^2 (N_2*r).^2 (N_2*log(1+c)).^3 (N_2*r).^3 (N_2*log(1+c)).^4 (N_2*r).^4 zscore(X) X_d pop];
            
            ZZZf = [N*log(1+c) N*r (N*log(1+c)).^2 (N*r).^2 (N*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 N_2*log(1+c) N_2*r (N_2*log(1+c)).^2 (N_2*r).^2 (N_2*log(1+c)).^3 (N_2*r).^3 (N_2*log(1+c)).^4 (N_2*r).^4];
        else if instruments==1
                Z = [N*log(1+c(:,2)) N*r (N*log(1+c(:,2))).^2 (N*r).^2 (N*log(1+c(:,2))).^3 (N*r).^3 (N*log(1+c(:,2))).^4 (N*r).^4 N_2*log(1+c(:,2)) N_2*r (N_2*log(1+c(:,2))).^2 (N_2*r).^2 (N_2*log(1+c(:,2))).^3 (N_2*r).^3 (N_2*log(1+c(:,2))).^4 (N_2*r).^4 zscore(X) X_d pop];
                
                ZZZf = [N*log(1+c(:,2)) N*r (N*log(1+c(:,2))).^2 (N*r).^2 (N*log(1+c(:,2))).^3 (N*r).^3 (N*log(1+c(:,2))).^4 (N*r).^4 N_2*log(1+c(:,2)) N_2*r (N_2*log(1+c(:,2))).^2 (N_2*r).^2 (N_2*log(1+c(:,2))).^3 (N_2*r).^3 (N_2*log(1+c(:,2))).^4 (N_2*r).^4];
            else if instruments==2
                Z = [N*log(1+c) (N*log(1+c)).^2 (N*log(1+c)).^3 (N*log(1+c)).^4 N_2*log(1+c) (N_2*log(1+c)).^2 (N_2*log(1+c)).^3 (N_2*log(1+c)).^4 zscore(X) X_d pop];
                
                ZZZf = [N*log(1+c) (N*log(1+c)).^2 (N*log(1+c)).^3 (N*log(1+c)).^4 N_2*log(1+c) (N_2*log(1+c)).^2 (N_2*log(1+c)).^3 (N_2*log(1+c)).^4];
                else if instruments==3
                    Z = [N*r (N*r).^2 (N*r).^3 (N*r).^4 N_2*r (N_2*r).^2 (N_2*r).^3 (N_2*r).^4 zscore(X) X_d pop];
        
                    ZZZf = [N*r (N*r).^2 (N*r).^3 (N*r).^4 N_2*r (N_2*r).^2 (N_2*r).^3 (N_2*r).^4];
                    else if instruments==4
                        Z = [N*log(1+c(:,1)) (N*log(1+c(:,1))).^2 (N*log(1+c(:,1))).^3 (N*log(1+c(:,1))).^4 N_2*log(1+c(:,1)) (N_2*log(1+c(:,1))).^2 (N_2*log(1+c(:,1))).^3 (N_2*log(1+c(:,1))).^4 zscore(X) X_d hpop];
        
                        ZZZf = [N*log(1+c(:,1)) (N*log(1+c(:,1))).^2 (N*log(1+c(:,1))).^3 (N*log(1+c(:,1))).^4 N_2*log(1+c(:,1)) (N_2*log(1+c(:,1))).^2 (N_2*log(1+c(:,1))).^3 (N_2*log(1+c(:,1))).^4];
                    else 
                        Z = [N*log(1+c(:,2)) (N*log(1+c(:,2))).^2 (N*log(1+c(:,2))).^3 (N*log(1+c(:,2))).^4 N_2*log(1+c(:,2)) (N_2*log(1+c(:,2))).^2 (N_2*log(1+c(:,2))).^3 (N_2*log(1+c(:,2))).^4 zscore(X) X_d hpop];
        
                        ZZZf = [N*log(1+c(:,2)) (N*log(1+c(:,2))).^2 (N*log(1+c(:,2))).^3 (N*log(1+c(:,2))).^4 N_2*log(1+c(:,2)) (N_2*log(1+c(:,2))).^2 (N_2*log(1+c(:,2))).^3 (N_2*log(1+c(:,2))).^4];
                        end
                    end
                end
            end
        end
    end
end
 

%Life Quality Index
XX = [s.^2 NE*s zscore(X) X_d pop];       
YY =  zscore(p(:,3));

NN=N;
NN_2=N_2;

  if excl_viol==1
      Z = Z(~v,:);
      XX = XX(~v,:);
      YY = YY(~v,:);
      XXXf = XXXf(~v,:);
      ZZZf = ZZZf(~v,:);
      n = size(YY,1);
      NN=(N(~v,:))';
      NN=(NN(~v,:))';
      NN_2=(N_2(~v,:))';
      NN_2=(NN_2(~v,:))';
  end
    
%IV
Q= XX'*Z*(inv(Z'*Z))*Z';
coeff_IV_Prosp=(Q*XX)\(Q*YY);             % estimation of the coefficients

    % standard errors
    resid=YY-XX*coeff_IV_Prosp;

SPEC(:,2)=resid;


%Public Utilities Coverage
XX = [s.^2 NE*s zscore(X) X_d pop];       
YY =  zscore(p(:,2));

NN=N;
NN_2=N_2;

  if excl_viol==1
      Z = Z(~v,:);
      XX = XX(~v,:);
      YY = YY(~v,:);
      XXXf = XXXf(~v,:);
      ZZZf = ZZZf(~v,:);
      n = size(YY,1);
      NN=(N(~v,:))';
      NN=(NN(~v,:))';
      NN_2=(N_2(~v,:))';
      NN_2=(NN_2(~v,:))';
  end
    
%IV
Q= XX'*Z*(inv(Z'*Z))*Z';
coeff_IV_Prosp=(Q*XX)\(Q*YY);             % estimation of the coefficients

    % standard errors
    resid=YY-XX*coeff_IV_Prosp;

SPEC(:,3)=resid;


%Not in Poverty
XX = [s.^2 NE*s zscore(X) X_d pop];       
YY =  zscore(p(:,4));

NN=N;
NN_2=N_2;

  if excl_viol==1
      Z = Z(~v,:);
      XX = XX(~v,:);
      YY = YY(~v,:);
      XXXf = XXXf(~v,:);
      ZZZf = ZZZf(~v,:);
      n = size(YY,1);
      NN=(N(~v,:))';
      NN=(NN(~v,:))';
      NN_2=(N_2(~v,:))';
      NN_2=(NN_2(~v,:))';
  end
    
%IV
Q= XX'*Z*(inv(Z'*Z))*Z';
coeff_IV_Prosp=(Q*XX)\(Q*YY);             % estimation of the coefficients

    % standard errors
    resid=YY-XX*coeff_IV_Prosp;

SPEC(:,4)=resid;


%Secondary Enrollment
XX = [s.^2 NE*s zscore(X) X_d pop];       
YY =  zscore(p(:,1));

NN=N;
NN_2=N_2;

  if excl_viol==1
      Z = Z(~v,:);
      XX = XX(~v,:);
      YY = YY(~v,:);
      XXXf = XXXf(~v,:);
      ZZZf = ZZZf(~v,:);
      n = size(YY,1);
      NN=(N(~v,:))';
      NN=(NN(~v,:))';
      NN_2=(N_2(~v,:))';
      NN_2=(NN_2(~v,:))';
  end
    
%IV
Q= XX'*Z*(inv(Z'*Z))*Z';
coeff_IV_Prosp=(Q*XX)\(Q*YY);             % estimation of the coefficients

    % standard errors
    resid=YY-XX*coeff_IV_Prosp;

SPEC(:,5)=resid;

%Network Centrality Measures
    %Betweenness Centrality
    SPEC(:,6) = V(:,2);

    %Bonachich Centrality
    SPEC(:,7) = V(:,3);
    
    %Local Clustering
    SPEC(:,8) = (V(:,4));

%%
%Correlation Table
disp('Correlations between Residuals and Network Centrality Statistics')
disp(corr(SPEC))

%Betweennes:

    %BR
    XX = [ones(n,1) SPEC(:,6)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,1));

    resid = SPEC(:,1)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between Best Response Residuals and Betweenness Centrality')
    disp(bbeta(2)*std(SPEC(:,6))/std(SPEC(:,1)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))

    %LQI
    XX = [ones(n,1) SPEC(:,6)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,2));

    resid = SPEC(:,2)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between LQI Residuals and Betweenness Centrality')
    disp(bbeta(2)*std(SPEC(:,6))/std(SPEC(:,2)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))

    %Utilitites Coverage
    XX = [ones(n,1) SPEC(:,6)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,3));

    resid = SPEC(:,3)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between Utilities Coverage Residuals and Betweenness Centrality')
    disp(bbeta(2)*std(SPEC(:,6))/std(SPEC(:,3)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))

    %Not in Poverty
	XX = [ones(n,1) SPEC(:,6)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,4));

    resid = SPEC(:,4)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between % Not in Poverty Residuals and Betweenness Centrality')
    disp(bbeta(2)*std(SPEC(:,6))/std(SPEC(:,4)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))

    %Secondary Enrollment
    XX = [ones(n,1) SPEC(:,6)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,5));

    resid = SPEC(:,5)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between Secondary Enrollment Residuals and Betweenness Centrality')
    disp(bbeta(2)*std(SPEC(:,6))/std(SPEC(:,5)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))


%Bonacich:
    
    %BR
    XX = [ones(n,1) SPEC(:,7)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,1));

    resid = SPEC(:,1)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between Best Response Residuals and Bonacich Centrality')
    disp(bbeta(2)*std(SPEC(:,7))/std(SPEC(:,1)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))

    %LQI
    XX = [ones(n,1) SPEC(:,7)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,2));

    resid = SPEC(:,2)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between LQI Residuals and Bonacich Centrality')
    disp(bbeta(2)*std(SPEC(:,7))/std(SPEC(:,2)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))

    %Utilitites Coverage
    XX = [ones(n,1) SPEC(:,7)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,3));

    resid = SPEC(:,3)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between Utilities Coverage Residuals and Bonacich Centrality')
    disp(bbeta(2)*std(SPEC(:,7))/std(SPEC(:,3)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))

    %Not in Poverty
	XX = [ones(n,1) SPEC(:,7)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,4));

    resid = SPEC(:,4)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between % Not in Poverty Residuals and Bonacich Centrality')
    disp(bbeta(2)*std(SPEC(:,7))/std(SPEC(:,4)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))
    
    %Secondary Enrollment
    XX = [ones(n,1) SPEC(:,7)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,5));

    resid = SPEC(:,5)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between Secondary Enrollment Residuals and Bonacich Centrality')
    disp(bbeta(2)*std(SPEC(:,7))/std(SPEC(:,5)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))
    
    
%Local Clustering    
    %BR
    XX = [ones(n,1) SPEC(:,8)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,1));

    resid = SPEC(:,1)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between Best Response Residuals and Local Clustering ')
    disp(bbeta(2)*std(SPEC(:,8))/std(SPEC(:,1)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))

    %LQI
    XX = [ones(n,1) SPEC(:,8)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,2));

    resid = SPEC(:,2)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between LQI Residuals and Local Clustering ')
    disp(bbeta(2)*std(SPEC(:,8))/std(SPEC(:,2)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))

    %Utilitites Coverage
    XX = [ones(n,1) SPEC(:,8)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,3));

    resid = SPEC(:,3)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between Utilities Coverage Residuals and Local Clustering ')
    disp(bbeta(2)*std(SPEC(:,8))/std(SPEC(:,3)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))

    %Not in Poverty
	XX = [ones(n,1) SPEC(:,8)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,4));

    resid = SPEC(:,4)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between % Not in Poverty Residuals and Local Clustering ')
    disp(bbeta(2)*std(SPEC(:,8))/std(SPEC(:,4)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))
    
    %Secondary Enrollment
    XX = [ones(n,1) SPEC(:,8)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,5));

    resid = SPEC(:,5)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt(var(2,2));

    disp('Correlation between Secondary Enrollment Residuals and Bonacich Centrality')
    disp(bbeta(2)*std(SPEC(:,8))/std(SPEC(:,5)))

    disp('p-value for the correlation')
    disp(2*(1-normcdf(abs(bbeta(2)/std_bbeta))))
    
    
 %Betweenness, Bonacich, Local Clustering   
    %BR
    XX = [ones(n,1) SPEC(:,6:8)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,1));

    resid = SPEC(:,1)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt([var(2,2) var(3,3) var(4,4)]);

    disp('Correlation between Best Response Residuals and All Centrality')
    disp(bbeta(2:4)'.*(std(SPEC(:,6:8))./std(SPEC(:,1))))

    disp('p-value of F-test for joint significance')
    SSRr = sum((SPEC(:,1)- ones(n,1)*mean(SPEC(:,1))).^2);
    SSRur = sum(resid.^2);
    F_test = ((n-size(XX,2))/(size(XX,2)-1))*(SSRr - SSRur)/SSRur;
    disp(1-fcdf(F_test,size(ones(n,1),2),(n-size(XX,2))))

    %LQI
    XX = [ones(n,1) SPEC(:,6:8)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,2));

    resid = SPEC(:,2)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt([var(2,2) var(3,3) var(4,4)]);

    disp('Correlation between LQI Residuals and All Centrality')
    disp(bbeta(2:4)'.*(std(SPEC(:,6:8))./std(SPEC(:,2))))

    disp('p-value of F-test for joint significance')
    SSRr = sum((SPEC(:,2)- ones(n,1)*mean(SPEC(:,2))).^2);
    SSRur = sum(resid.^2);
    F_test = ((n-size(XX,2))/(size(XX,2)-1))*(SSRr - SSRur)/SSRur;
    disp(1-fcdf(F_test,size(ones(n,1),2),(n-size(XX,2))))

    %Utilitites Coverage
    XX = [ones(n,1) SPEC(:,6:8)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,3));

    resid = SPEC(:,3)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt([var(2,2) var(3,3) var(4,4)]);

    disp('Correlation between Utilities Coverage Residuals and All Centrality')
    disp(bbeta(2:4)'.*(std(SPEC(:,6:8))./std(SPEC(:,3))))

    disp('p-value of F-test for joint significance')
    SSRr = sum((SPEC(:,3)- ones(n,1)*mean(SPEC(:,3))).^2);
    SSRur = sum(resid.^2);
    F_test = ((n-size(XX,2))/(size(XX,2)-1))*(SSRr - SSRur)/SSRur;
    disp(1-fcdf(F_test,size(ones(n,1),2),(n-size(XX,2))))

    %Not in Poverty
	XX = [ones(n,1) SPEC(:,6:8)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,4));

    resid = SPEC(:,4)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt([var(2,2) var(3,3) var(4,4)]);

    disp('Correlation between % Not in Poverty Residuals and All Centrality')
    disp(bbeta(2:4)'.*(std(SPEC(:,6:8))./std(SPEC(:,4))))

    disp('p-value of F-test for joint significance')
    SSRr = sum((SPEC(:,4)- ones(n,1)*mean(SPEC(:,4))).^2);
    SSRur = sum(resid.^2);
    F_test = ((n-size(XX,2))/(size(XX,2)-1))*(SSRr - SSRur)/SSRur;
    disp(1-fcdf(F_test,size(ones(n,1),2),(n-size(XX,2))))
    
    %Secondary Enrollment
    XX = [ones(n,1) SPEC(:,6:8)];

    bbeta = (XX'*XX)\(XX'*SPEC(:,5));

    resid = SPEC(:,5)-XX*bbeta;

    var = (std(resid)^2)*(inv(XX'*XX));
    std_bbeta = sqrt([var(2,2) var(3,3) var(4,4)]);

    disp('Correlation between Secondary Enrollment Residuals and All Centrality')
    disp(bbeta(2:4)'.*(std(SPEC(:,6:8))./std(SPEC(:,5))))

    disp('p-value of F-test for joint significance')
    SSRr = sum((SPEC(:,5)- ones(n,1)*mean(SPEC(:,5))).^2);
    SSRur = sum(resid.^2);
    F_test = ((n-size(XX,2))/(size(XX,2)-1))*(SSRr - SSRur)/SSRur;
    disp(1-fcdf(F_test,size(ones(n,1),2),(n-size(XX,2))))
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    





